/* 
 * Modified by 
 * Dmitriy Cherkashin 
 * dch@ucrouter.ru
 * 2002
 */

/*************************************************************************/
/*                                                                       */
/* FILE NAME                                      VERSION                */
/*                                                                       */
/*     DMA.c                       KS32C5000, KS32C50100   : version 1.0 */
/*                                                                       */
/* COMPONENT                                                             */
/*                                                                       */
/*                                                                       */
/*                                                                       */
/* DESCRIPTION                                                           */
/*                                                                       */
/*                                                                       */
/* AUTHOR                                                                */
/*                                                                       */
/*      Young Sun KIM, Samsung Electronics, Inc.                         */
/*                                                                       */
/* DATA STRUCTURES                                                       */
/*                                                                       */
/*                                                                       */
/* FUNCTIONS                                                             */
/*                                                                       */
/*      Evalution code for DMA control block                             */
/*                                                                       */
/* DEPENDENCIES                                                          */
/*                                                                       */
/*                                                                       */
/* HISTORY                                                               */
/*                                                                       */
/*         NAME            DATE                    REMARKS               */
/*                                                                       */
/*         in4maker     12-18-1998      timer function updated           */
/*************************************************************************/
#include "ks32c50.h"
#include "evm50100.h"

#define GDMA_DEBUG   0           	/* DEBUG mode */

#define GDMATestSize       0x1000 	/* Default Test Size 4KB */   
U8      GDMATestSrc[GDMATestSize];	/* Default source buffer */
U8      GDMATestDst[GDMATestSize];	/* Default destination buffer */

/* global variable */
volatile U32 Gdma0DoneFlag = 0;   // GDMA Channel 0 Transfer done interrupt flag
volatile U32 Gdma1DoneFlag = 0;   // GDMA Channel 1 Transfer done interrupt flag


//////////////////////////////////////////////////////////////////////////////// 
// GDMA0, GDMA1 Interrupt Service Routines /////////////////////////////////////
//////////////////////////////////////////////////////////////////////////////// 

void  GDMA0isr(void)
{
#if GDMA_DEBUG
 Print("\nTRANSFER MODE IS ");
 switch( (GDMACON0&GDMACON_MODE) ) 
  {
   case GDMACON_MODE_MEMMEM  : Print("\nSOFTWARE."); break;
   case GDMACON_MODE_EXTDREQ : Print("\nEXTDREQ."); break;
   case GDMACON_MODE_U0      : Print("\nUART0."); break;
   case GDMACON_MODE_U1      : Print("\nUART1.");
  }
 GdmaReset(0);
#endif
 Gdma0DoneFlag = 1; 
}

void  GDMA1isr(void)
{
#if GDMA_DEBUG
 Print("\nTRANSFER MODE IS ");
 switch( (GDMACON1&GDMACON_MODE) ) 
  {
   case GDMACON_MODE_MEMMEM  : Print("\nSOFTWARE."); break;
   case GDMACON_MODE_EXTDREQ : Print("\nEXTDREQ."); break;
   case GDMACON_MODE_U0      : Print("\nUART0."); break;
   case GDMACON_MODE_U1      : Print("\nUART1.");
  }
 GdmaReset(1);
#endif
 Gdma1DoneFlag = 1; 
}


//////////////////////////////////////////////////////////////////////////////// 
// GDMA Configuration viewer /////////////////////////////////////////////////// 
//////////////////////////////////////////////////////////////////////////////// 

static void GDMAConfigView(int cn)
{
 U32 gdmacon;			// GDMA channel cn control register
 U32 gdmasrc;			// GDMA channel cn source address register 
 U32 gdmadst;			// GDMA channel cn destination address register 
 U32 gdmacnt;			// GDMA channel cn count register 

 if(cn == 1) 
  { 
// GDMA Channel 1 //////////////////////////////////////////////////////////////
   gdmacon = GDMACON1;			// GDMA 1 Control register
   gdmasrc = GDMASRC1;			// GDMA 1 Source address register 
   gdmadst = GDMADST1;			// GDMA 1 Destination address register 
   gdmacnt = GDMACNT1;			// GDMA 1 Count Register 
  }
 else 
 if(cn == 0)
  {
// GDMA Channel 0 ////////////////////////////////////////////////////////////// 
   gdmacon = GDMACON0;			// GDMA 0 Control Register 
   gdmasrc = GDMASRC0;			// GDMA 0 Source Address Register 
   gdmadst = GDMADST0;			// GDMA 0 Destination Address Register 
   gdmacnt = GDMACNT0;			// GDMA 0 Count Register 
  }
 else
  { 
   return;
  } 

//////////////////////////////////////////////////////////////////////////////// 
// GDMACON ///////////////////////////////////////////////////////////////////// 
//////////////////////////////////////////////////////////////////////////////// 

 Print("\nGDMA%d Control Register...................0x%08x", cn, gdmacon);

// GDMA_RE // [0] Run enable/disable (RE) ////////////////////////////////////// 

 Print("\n[0]     GDMA%d Run enable/disable (RE)....%s", cn,
  ((gdmacon & GDMACON_RE) !=0)  ?
   "1(Enable)"	:
   "0(Disable)"
  );

//////////////////////////////////////////////////////////////////////////////// 
// GDMA_BS // [1] Busy status (BS) /////////////////////////////////////////////
//////////////////////////////////////////////////////////////////////////////// 

 Print("\n[1]     GDMA%d Busy status (BS)...........%s", cn,
  ((gdmacon & GDMACON_BS) != 0)	? 
   "1(DMA is active)"		:
   "0(DMA is idle)"
  );

//////////////////////////////////////////////////////////////////////////////// 
// [3:2] GDMA mode selection (MODE) ////////////////////////////////////////////
//////////////////////////////////////////////////////////////////////////////// 

 Print("\n[3:2]   GDMA%d mode selection (MODE)......",cn);
 switch(gdmacon & GDMACON_MODE)
 {
  case GDMACON_MODE_EXTDREQ: Print("01(External DMA Request mode)"); break;
  case GDMACON_MODE_U0     : Print("10(UART0 Mode)"); break;
  case GDMACON_MODE_U1     : Print("11(UART1 Mode)"); break;
  case GDMACON_MODE_MEMMEM : 
  default 		   : 
   Print("00(Memory to Memory)"); 
   break;
 }

//////////////////////////////////////////////////////////////////////////////// 
// [4] Destination address direction (GDMA_DST_DEC) ////////////////////////////
//////////////////////////////////////////////////////////////////////////////// 

 Print("\n[4]     GDMA%d Dest. address direction....%s", cn,  
  ((gdmacon & GDMACON_DST_DEC) != 0) ?
   "1(Decrement)"	:
   "0(Increment)"
 ); 


//////////////////////////////////////////////////////////////////////////////// 
// [5] Source address direction (GDMA_SRC_DEC) /////////////////////////////////
//////////////////////////////////////////////////////////////////////////////// 

 Print("\n[5]     GDMA%d Source address direction...%s", cn,
  ((gdmacon & GDMACON_SRC_DEC) != 0)	? 
   "1(decrement)"	:
   "0(Increment)"
 );
  
//////////////////////////////////////////////////////////////////////////////// 
// [6] Destination address fix (GDMA_DST_FIX) //////////////////////////////////
//////////////////////////////////////////////////////////////////////////////// 

 Print("\n[6]     GDMA%d Destination address fix....%s", cn,
  ((gdmacon & GDMACON_DST_FIX) != 0)	? 
   "1(Not change address)" :
   "0(Increase/Decrease address)"
 );


//////////////////////////////////////////////////////////////////////////////// 
// [7] Source address fix (GDMA_SRC_FIX) ///////////////////////////////////////
//////////////////////////////////////////////////////////////////////////////// 

 Print("\n[7]     GDMA%d Source address fix.........%s", cn,
  ((gdmacon & GDMACON_SRC_FIX) != 0)	? 
   "1(Not change address)" :	
   "0(Increase/Decrease)"
 );

//////////////////////////////////////////////////////////////////////////////// 
// [8] Stop interrupt enable (GDMA_SI) /////////////////////////////////////////
//////////////////////////////////////////////////////////////////////////////// 

 Print("\n[8]     GDMA%d Stop interrupt enable(SI)..%s", cn,
  ((gdmacon & GDMACON_SI) != 0)		? 
   "1(Generate stop interrupt)" :
   "0(do not generate stop int)"
 );
  

//////////////////////////////////////////////////////////////////////////////// 
// [9] Four-data burst enable (GDMA_FB) ////////////////////////////////////////
//////////////////////////////////////////////////////////////////////////////// 

 Print("\n[9]     GDMA%d Four-data burst enable(FB).%s", cn,
  ((gdmacon & GDMACON_FB) != 0)		?	 
   "1(Enable)"	:
   "0(Disable)"
  );

//////////////////////////////////////////////////////////////////////////////// 
// [10] Transfer direction (for UART0/UART1 only) (GDMA_TD) ////////////////////
//////////////////////////////////////////////////////////////////////////////// 

 Print("\n[10]    GDMA%d Transfer direction.........%s", cn,
  ((gdmacon & GDMACON_TD) == GDMACON_TD_MEMTOU)	?
   "1(Memory to UART0/UART1)"	:
   "0(UART0/UART1 to memory)"
 );

//////////////////////////////////////////////////////////////////////////////// 
// [11] Single/Block mode (GDMA_SB) ////////////////////////////////////////////
//////////////////////////////////////////////////////////////////////////////// 

 Print("\n[11]    GDMA%d Single/Block mode (SB).....%s", cn,
  ((gdmacon & GDMACON_SB_BLOCK) != 0) 	?
   "1(Block mode)"	:
   "0(Single Mode)"
 );

//////////////////////////////////////////////////////////////////////////////// 
// [13:12] Transfer width (TW) /////////////////////////////////////////////////
//////////////////////////////////////////////////////////////////////////////// 

 Print("\n[13:12] GDMA%d Transfer width (TW)........",cn);
 switch( (gdmacon & GDMACON_TW) )	// Transfer Width 
  {
   case GDMACON_TW_BYTE     : Print("00(byte)");     break;
   case GDMACON_TW_HALFWORD : Print("01(halfword)"); break;
   case GDMACON_TW_WORD     : Print("10(word)");     break;
   default                  : Print("11(unused)");   break;
  }


//////////////////////////////////////////////////////////////////////////////// 
// [14] Continuous mode (GDMA_CM) //////////////////////////////////////////////
//////////////////////////////////////////////////////////////////////////////// 

 Print("\n[14]    GDMA%d Continuous mode (CM).......%s", cn, 
  ((gdmacon & GDMACON_CM) != 0)	? 	// Continuos Mode  
   "1(Continuous mode)"	:
   "0(Normal operation)"
 );

//////////////////////////////////////////////////////////////////////////////// 
// [15] Demand mode (GDMA_DM) //////////////////////////////////////////////////
//////////////////////////////////////////////////////////////////////////////// 

 Print("\n[15]    GDMA%d Demand mode (DM)...........%s", cn,
  ((gdmacon & GDMACON_DM) != 0)	? 	// Demand Mode 
   "1(Demand mode)"	:
   "0(Normal operation)"
 );

//
 Print("\nGDMASRC%d Source Address Register.........0x%08x", cn, gdmasrc);
 Print("\nGDMADST%d Dest. Address Register..........0x%08x", cn, gdmadst);
 Print("\nGDMACNT%d Transfer Count Register.........0x%08x", cn, gdmacnt);
 Print("\n");
}


//////////////////////////////////////////////////////////////////////////////// 
// Initialize GDMA control register ////////////////////////////////////////////
//////////////////////////////////////////////////////////////////////////////// 

void GdmaReset(unsigned gdma_channel)
{
 if(gdma_channel == 1) 
  {
// GDMA Channel 1 reset ////////////////////////////////////////////////////////   
   GDMASRC1 = 0;			// GDMA 1 source address 
   GDMADST1 = 0;			// GDMA 1 destination address 
   GDMACNT1 = 0;			// GDMA 1 transfer count register 
   GDMACON1 = 0;			// GDMA 1 control register 
  }
 else 
 if(gdma_channel == 0)
  {
// GDMA Channel 0 reset ////////////////////////////////////////////////////////   
   GDMASRC0 = 0;			// GDMA 0 source address 
   GDMADST0 = 0;			// GDMA 0 destination address 
   GDMACNT0 = 0;			// GDMA 0 transfer count register 
   GDMACON0 = 0;			// GDMA 0 control register 
  }
}


//////////////////////////////////////////////////////////////////////////////// 
// GDMA Registers Read & write test ////////////////////////////////////////////
//////////////////////////////////////////////////////////////////////////////// 

// print GDMA source address register ////////////////////////////////////////// 
#define PrintGDMASRC(cn)	 			\
 Print("\nGDMASRC%d           [0x%08x] = 0x%08x", 	\
  cn				,			\
 (cn ? &GDMASRC1 : &GDMASRC0)	,			\
 (cn ?  GDMASRC1  : GDMASRC0)  				\
); 

// print GDMA destination address register ///////////////////////////////////// 
#define PrintGDMADST(cn) 				\
 Print("\nGDMADST%d           [0x%08x] = 0x%08x", 	\
  cn				,			\
 (cn ? &GDMADST1 : &GDMADST0)	, 			\
 (cn ?  GDMADST1 :  GDMADST0) 				\
); 

// print GDMA transfer count register ////////////////////////////////////////// 
#define PrintGDMACNT(cn) 				\
 Print("\nGDMACNT%d           [0x%08x] = 0x%08x",	\
  cn				,			\
 (cn ? &GDMACNT1 : &GDMACNT0)	, 			\
 (cn ?  GDMACNT1 :  GDMACNT0)				\
); 

// print GDMA control register ///////////////////////////////////////////////// 
#define PrintGDMACON(cn) 				\
 Print("\nGDMACON%d           [0x%08x] = 0x%08x",  	\
  cn				,			\
 (cn ? &GDMACON1 : &GDMACON0) 	, 			\
 (cn ?  GDMACON1 :  GDMACON0)				\
); 

//////////////////////////////////////////////////////////////////////////////// 
// GDMA Interrupt enable & disable and clear transfer done flag ////////////////
//////////////////////////////////////////////////////////////////////////////// 

void GdmaIntEnable(unsigned gdma_channel)
{
 if(gdma_channel == 1) 
  {
// GDMA channel 1 interrupt enable /////////////////////////////////////////////  
   Gdma1DoneFlag = 0; 			// clear GDMA1 done flag	
   Enable_Int(nGDMA1_INT);		// enable GDMA1 INT  
  }
 else 
 if(gdma_channel == 0)
  {
// GDMA channel 0 interrupt enable /////////////////////////////////////////////
   Gdma0DoneFlag = 0;			// clear GDMA1 done flag   
   Enable_Int(nGDMA0_INT);		// enable GDMA0 INT 
  } 
// 
 Enable_Int(nGLOBAL_INT);		// global interrupt enable 
}


void GdmaIntDisable(unsigned gdma_channel)
{
 if(gdma_channel == 1) 
  {
// disable GDMA channel 1 interrupt ////////////////////////////////////////////   
   Disable_Int(nGDMA1_INT);		// disable GDMA1_INT
   Gdma1DoneFlag = 0;			// clear GDMA1 done flag 
  }
 else 
 if(gdma_channel == 0)
  {
// disable GDMA channel 0 interrupt ////////////////////////////////////////////
   Disable_Int(nGDMA0_INT);		// disable GDMA0_INT 	
   Gdma0DoneFlag = 0; 			// clear GDMA0 done flag 
  }
}

//////////////////////////////////////////////////////////////////////////////// 
// Initialize GDMA Memory with ASCII pattern for test //////////////////////////
//////////////////////////////////////////////////////////////////////////////// 

//
// src   = source size 
// tsize = size in bytes 
// 

static void GdmaMemInit(void *src, void * dst, int tsize)
{
 U8 * srcptr;
 U8 * dstptr;
 int ascii = 48;
 int cnt;

 srcptr = (U8 *) src;
 dstptr = (U8 *) dst;
 cnt = tsize;

 while(cnt--) 
  {
   if(ascii > 123) 
    {
     ascii = 48;
    }

   *srcptr++ = ascii++;
   *dstptr++ = 0;
  }
}

//
// src   = source address 
// dst   = destination address
// tsize = size in bytes
// 

static int GdmaMemComp(void *src, void *dst, int tsize)
{
 U8 * SrcAddr; 
 U8 * DstAddr;
 int  cnt;

 SrcAddr = (U8 *)src;				/* byte pointer to source */
 DstAddr = (U8 *)dst;				/* byte pointer to destination */
 cnt	 = tsize;				/* byte count */

 while(cnt--)
  {
   if ( *DstAddr != *SrcAddr )
    {
     Print("\nMemory Byte Test Error") ;
     Print("\nSource Data is %02x at Addr %08x", *SrcAddr, SrcAddr);
     Print("\nDestination Data is %02x at Addr %08x", *DstAddr, DstAddr);
     return(0);
    }
   DstAddr++;
   SrcAddr++;
  }
// 
 return(1);
}


//////////////////////////////////////////////////////////////////////////////// 
// GDMA Memory to Memory test Module ///////////////////////////////////////////
//////////////////////////////////////////////////////////////////////////////// 

void GdmaMemMemTest(void)
{
 U32 gdmacon;  		 		// control register
 U32 gdmasrc;   			// source address register
 U32 gdmadst;   			// destination address register
 U32 gdmacnt;   			// counter register
 int channel;				// GDMA channel 
 int repeat;				// repeat count   
 int bsize;				// byte count
 int rvl;				// 
 U8 it;
 int j;					// repeat num

//
 rvl = 1;

 Print("\n\nGDMA memory to memory test");

// Get GDMA Channel ////////////////////////////////////////////////////////////
 while(1) 
  { 
   Print("\nSelect GDMA Channel                      [0/1/Q]");
   channel = get_upper();
   if((channel == '1') || (channel == '0'))
    {
     channel -= '0';
     break;
    }
   else
   if(channel == 'Q')
    {
     return;
    }  
  }
       
// Setup memory to memory gdma transfer ////////////////////////////////////////

////////////////////////////////////////////////////////////////////////////////
// destination address /////////////////////////////////////////////////////////
////////////////////////////////////////////////////////////////////////////////

 gdmadst = (U32) GDMATestDst;  		// default destination addr
 Print("\nChange Default Dest. Address (0x%08x)[Y/N/Q]", gdmadst);
 it = get_upper();
 if(it == 'Q')
  {
   return;
  }
 else  
 if(it == 'Y')
  {
// change destination address //////////////////////////////////////////////////
   Print("\nInput Destination Address              0x");
   gdmadst = get_number(16,0);	
   if(gdmadst == 0) 
    {
     gdmadst = (U32) GDMATestDst;  	// default destination addr
    }
  }

  
////////////////////////////////////////////////////////////////////////////////
// source address //////////////////////////////////////////////////////////////
////////////////////////////////////////////////////////////////////////////////

 gdmasrc = (U32) GDMATestSrc;  		// default source addr
 Print("\nChange Default Src. Address (0x%08x) [Y/N/Q]", gdmasrc);
 it = get_upper();
 if(it == 'Q')
  {
   return;
  } 
 else
 if(it == 'Y')
  {
// change default destination address //////////////////////////////////////////
   Print("\nInput Source Address                   0x");
   gdmasrc = get_number(16,0);
   if(gdmasrc == 0) 
    {
     gdmasrc = (U32) GDMATestSrc;  	// default source addr
    }
  }

////////////////////////////////////////////////////////////////////////////////
// memory size (in bytes) //////////////////////////////////////////////////////
////////////////////////////////////////////////////////////////////////////////

 gdmacnt = (U32) GDMATestSize;
 Print("\nChange Default Size (0x%08x)         [Y/N/Q]", gdmacnt);
 it = get_upper();
 if(it == 'Q')
  {
   return;
  }
 else   
 if(it == 'Y')
  {
// change default test size ////////////////////////////////////////////////////
   Print("\nInput Memory Test Size                 0x");
   gdmacnt = get_number(16,0);
   if(gdmacnt == 0) 
    {
     gdmacnt = (U32) GDMATestSize;
    }
// 
   if(gdmacnt < 4)
    {
     gdmacnt = 4; 
    }  
   gdmacnt = (gdmacnt / 4) * 4;
  }  

///////////////////////////////////////////////////////////////////////////////
 if(channel == 0)
  {
   gdmacon = GDMACON0;  
  }
 else
  {
   gdmacon = GDMACON1;
  } 

//
 bsize = gdmacnt; 			// size in bytes

///////////////////////////////////////////////////////////////////////////////
// repeat count ///////////////////////////////////////////////////////////////
///////////////////////////////////////////////////////////////////////////////

 Print("\nInput Memory Test repeat times    (5) 0x");
 repeat = get_number(16,0);
 if(repeat == 0) 
  {
   repeat = 5;
  }

//
 Print("\n");
 //.................................................
 Print("\nSelected GDMA channel             : GDMA%d", channel);
 Print("\nGDMA%d source address              : 0x%08x", channel, gdmasrc);
 Print("\nGDMA%d destination address         : 0x%08x", channel, gdmadst);
 Print("\nGDMA%d memory test size            : 0x%08x", channel, gdmacnt);
 Print("\nGDMA%d Mem test repeat times       : 0x%08x", channel, repeat);
 Print("\n");

 for(j = 0; j < (repeat * 3); j++) 
  {
// Transfer width will be changed to 8,16,32 bit 
   gdmacon &= ~GDMACON_TW;
   if((j%3) == 0)
    { 
// transfer width = byte ///////////////////////////////////////////////////////
     gdmacon |= GDMACON_TW_BYTE;     	// Transfer Width = byte (8bit)
     gdmacnt = bsize;			// Transfer count = bytes count
    }
   else
   if((j%3) == 1)
    {
// transfer width = half word //////////////////////////////////////////////////
     gdmacon |= GDMACON_TW_HALFWORD; 	// Transfer Width = half word (16 bit)
     gdmacnt = bsize / 2;		// Transfer count = half word count
    }
   else
   if((j%3) == 2) 
    {
// transfer width = word ///////////////////////////////////////////////////////
     gdmacon |= GDMACON_TW_WORD;     	// Transfer Width = word
      gdmacnt = bsize / 4;		// Transfer count = word count
    }

// Initialize source memory with ascii pattern /////////////////////////////////
   GdmaMemInit((U32 *) gdmasrc, (U32 *) gdmadst, (int) bsize); 

// GDMA related register all clear to reset state //////////////////////////////
   GdmaReset(channel);  

   gdmacon         |=
   GDMACON_SI      |			// GDMA Stop Interrupt Enable	
   GDMACON_MODE_MEMMEM;  		// memory to memory

   gdmacon &= ~GDMACON_DST_DEC;		// clear destination decrement bit 
   gdmacon &= ~GDMACON_SRC_DEC;      	// clear source decrement bit 

   if(channel)
    { 
     Gdma1DoneFlag = 0;
    }
   else
    {
     Gdma0DoneFlag = 0;
    } 

// Enable GDMA Channel Interrupt ///////////////////////////////////////////////
   GdmaIntEnable(channel);		// enable interrupt

   if(channel == 1)
    {
// Set GDMA channel 1 registers ////////////////////////////////////////////////
     GDMASRC1 = gdmasrc;
     GDMADST1 = gdmadst;
     GDMACNT1 = gdmacnt;
     GDMACON1 = gdmacon;

     GDMA1_RUN_ENABLE = 1;

// Wait GDMA1 Done Interrupt /////////////////////////////////////////////////// 
     while(Gdma1DoneFlag == 0) { }
     Gdma1DoneFlag = 0;
    }
   else
    {
// Set GDMA channel 0 registers ////////////////////////////////////////////////
     GDMASRC0 = gdmasrc;
     GDMADST0 = gdmadst;
     GDMACNT0 = gdmacnt;
     GDMACON0 = gdmacon;

     GDMA0_RUN_ENABLE = 1;

// Wait GDMA0 Done Interrupt ///////////////////////////////////////////////////
     while(Gdma0DoneFlag == 0) { }
     Gdma0DoneFlag = 0;
    }  

// 
   Disable_Int(nGLOBAL_INT);		// disable global interrupt 
   GdmaReset(channel);

// Disable GDMA interrupt & clear interrupt Done flag //////////////////////////  
   GdmaIntDisable(channel);

   if(
     GdmaMemComp
      (
       (void *) gdmasrc,	 	// source address
       (void *) gdmadst,		// destination address
        bsize				// size in bytes 
       ) == 0
    ) 
    {
     rvl = 0;
     break;
    }
  }

 if(rvl == 0)
  {
   Print("\nFail.\n");
  }
 else  
  {
   Print("\nOk.\n");
  }
}

////////////////////////////////////////////////////////////////////////////////
// UART0/UART1 to memory test //////////////////////////////////////////////////
////////////////////////////////////////////////////////////////////////////////

static void UartToMemTest(void)
{
 int uart;				// UART number
 int gdma;				// gdma number
 Print("\n\nUART0/UART1 to memory transfer test");

// Get GDMA Channel ////////////////////////////////////////////////////////////
 while(1) 
  { 
   Print("\nSelect UART0/UART1 [0/1/Q]");
   uart = get_upper();			// get UART number from console
   if( uart == 'Q')
    {
     return;
    }
   else   
   if((uart == '1') || (uart == '0'))	
    {
     uart -= '0';
     break;
    }   				
  }

// Get GDMA Channel ////////////////////////////////////////////////////////////

 while(1) 
  { 
   Print("\nSelect GDMA Channel[0/1/Q]");
   gdma = get_upper();			// get GDMA channel number
   if(gdma == 'Q')
    {
     return;
    }
   else
   if( (gdma == '1') || (gdma == '0') )	
    {
     gdma -= '0';			
     break;
    }
  }

// GDMA Uart0 to Memory transfer test //////////////////////////////////////////
 UartToMemoryTest(uart, gdma);		// UART to memory test
}  

//////////////////////////////////////////////////////////////////////////////// 
// UART uart_channel to Memory test ////////////////////////////////////////////
//////////////////////////////////////////////////////////////////////////////// 

void UartToMemoryTest(int uart, int gdma)
{
 U32 gdmasrc;				// GDMA source address 
 U32 gdmadst;				// GDMA destination address
 U32 gdmacon;				// GDMA control register 
 U32 gdmacnt;				// GDMA transfer count register 
 U32 uartcont;				// saved content of UARTCONT1/UARTCONT0
 U8 * ptr;				// pointer in destination buffer
 int size;				// count of received data
 int changed;				// changed flag of destination buffer
 int i;					// position in destination buffer

// destination address not valid => set the destination address //////////////// 
 gdmadst = (U32) GDMATestDst; 		// default destination address
  
// Rx byte count ///////////////////////////////////////////////////////////////

 Print("\nReceived Data data count 0x");
 size = get_number(16,0);		// read size from console 
 if(size < 1)
  {
   size = 1;				// set size to low limit 
  }
 if(gdmacnt > GDMATestSize)
  {
   size = GDMATestSize;			// set size to upper limit 
  } 

// 
 gdmacnt = size;			// transfer count 

// Set destination buffer ////////////////////////////////////////////////////// 

 ptr = (U8 *) gdmadst;			// pointer in destination buffer 
 for(i = 0; i < size; ++i)
  {					// fill destination 
   *ptr++ = i; 				// buffer 
  }
 
// Print destination address, size /////////////////////////////////////////////
 Print("\nReceived data size = 0x%08x", gdmacnt); 	
 Print("\nGDMA DownLoad area = 0x%08x", gdmadst);

// GDMA related register all clear to reset state //////////////////////////////
 GdmaReset(gdma);  

// configure GDMA control register & UART control register /////////////////////

 gdmacon = 
  GDMACON_SI        | 			// GDMA Stop Interrup Enable 
  GDMACON_SRC_FIX   | 			// source address fix
  GDMACON_TD_UTONEM |			// 0 trasfer firection for UART mode    
  GDMACON_TW_BYTE; 			// 0 transfer width - byte

 if(uart == 1) 
  {
   gdmacon |= GDMACON_MODE_U1;		// mode = UART1 
   uartcont   =  UARTCONT1; 		// read UARTCONT1 content 
  } 
 else 
 if(uart == 0) 
  {
   gdmacon |= GDMACON_MODE_U0;		// mode = UART0
   uartcont   =  UARTCONT0; 		// read UARTCONT0 content 
  }


////////////////////////////////////////////////////////////////////////////////
// set GDMA registers //////////////////////////////////////////////////////////
////////////////////////////////////////////////////////////////////////////////

 uartcont &= ~UARTCON_RXM;     		// clear UART Rx mode bits 
 if(gdma == 1) 
  { 
   uartcont |= UARTCON_RXM_GDMA1REQ;	// Rx mode = GDMA1 request 

// set GDMA channel 1 registers //////////////////////////////////////////////// 
   GDMASRC1 = gdmasrc;			// GDMA 1 source address 
   GDMADST1 = gdmadst;			// GDMA 1 destination address	
   GDMACNT1 = gdmacnt;			// GDMA 1 transfer count 
   GDMACON1 = gdmacon;			// GDMA 1 control register
  }
 else 
 if(gdma == 0) 
  {
   uartcont |= UARTCON_RXM_GDMA0REQ;	// Rx mode = GDMA1 request 

// set GDMA channel 0 registers ////////////////////////////////////////////////
   GDMASRC0 = gdmasrc;			// GDMA 0 source address
   GDMADST0 = gdmadst;			// GDMA 0 destination address
   GDMACNT0 = gdmacnt;			// GDMA 0 transfer count 
   GDMACON0 = gdmacon;			// GDMA 0 control register 
  }

// print GDMA registers ////////////////////////////////////////////////////////

 PrintGDMASRC(gdma);			// GDMA source address 
 PrintGDMADST(gdma);			// GDMA destination address	
 PrintGDMACNT(gdma);			// GDMA transfer count 
 PrintGDMACON(gdma);			// GDMA control register
 Print("\n");

 if(uart == 1)
  {
   UARTCONT1 = uartcont;
  }
 else
 if(uart == 0)
  {
   UARTCONT0 = uartcont;
  }
  
// Enable GDMA Channel Interrupt, Clear Done Flag and Global enable interrupt //  
 GdmaIntEnable(gdma);
 Enable_Int(nGLOBAL_INT);		// global interrupt 

// Start GDMA Channel //////////////////////////////////////////////////////////

 if(gdma) 
  {
   Gdma1DoneFlag = 0;			// clear gdma1 interrupt done flag  
   GDMA1_RUN_ENABLE = 1;		// start GDMA1	
   while(Gdma1DoneFlag == 0) {;} 	// wait GDMA1 done interrupt 
  }
 else  
  {
   Gdma0DoneFlag = 0;			// clear gdma0 interrupt done flag 
   GDMA0_RUN_ENABLE = 1;		// start GDMA0 
   while(Gdma0DoneFlag == 0) {;}  	// wait GDMA0 done interrupt
  }

// 
 Disable_Int(nGLOBAL_INT);		// disable global interrupt 
 GdmaIntDisable(gdma);			// disable GDMA channel

// restore UART configuration ////////////////////////////////////////////////// 

 uartcont = (uartcont & ~UARTCON_RXM) | UARTCON_RXM_INTREQ;
 if(uart == 1)
  {
   UARTCONT1 = uartcont;		// restore UART1 control register
  } 
 else
 if(uart == 0)
  {
   UARTCONT0 = uartcont;		// restore UART0 control register
  } 

// check destination buffer ////////////////////////////////////////////////////
 ptr = (U8 *) gdmadst;
 changed = 0;
 for(i = 0; i < size; ++i)
  {
   if(*ptr++ != i)
    {
     changed = 1;
    } 
  }

 if(changed == 0)
  {
// destination buffer not changed //////////////////////////////////////////////

   Print("\nFail. Destination buffer not changed.\n");
  }
 else
  {
// Destination buffer changed //////////////////////////////////////////////////

   Print("\nPrint received buffer [Y/N]");  
   changed = get_upper();
   if(changed == 'Y')
    {
// print content of destination buffer /////////////////////////////////////////
     Print("\n");
     MemDump((U32 *) gdmadst, (U32 *) gdmadst + size);
    }   
   Print("\n");
  }
}

//////////////////////////////////////////////////////////////////////////////// 
// GDMA Memory to UART0/UART1 transfer test //////////////////////////////////// 
//////////////////////////////////////////////////////////////////////////////// 

static void MemToUartTest(void)
{
 int uart;				// UART number
 int gdma;				// GDMA Channel 

 Print("\n\nGDMA memory to UART0/UART1 transfer test");

// Get GDMA Channel ////////////////////////////////////////////////////////////
 while(1)
  { 
   Print("\nSelect UART0/UART1               [0/1/Q]");
   uart = get_upper();			// get UART number from console
   if(uart == 'Q')
    {
     return;
    }
   else
   if( (uart == '1') || (uart == '0') )
    {
     uart -= '0';
     break;
    }
  }

// Get GDMA Channel ////////////////////////////////////////////////////////////
 while(1) 
  { 
   Print("\nSelect GDMA Channel              [0/1/Q]");
   gdma = get_upper();			// 
   if(gdma == 'Q')
    {
     return;
    } 
   else
   if( (gdma == '1') || (gdma == '0') )
    {
     gdma -= '0';
     break;
    }
  }

// GDMA Memory to Uart transfer test ///////////////////////////////////////////
 MemoryToUartTest(uart, gdma); 	 	// memory to UART 0 test
}

//////////////////////////////////////////////////////////////////////////////// 
// memory to UART transfer test ////////////////////////////////////////////////
//////////////////////////////////////////////////////////////////////////////// 


void MemoryToUartTest(int uart, int gdma)
{
 U32 uartcont;				// UART0/UART1 control register content
 U32 gdmasrc;				// GDMA source address
 U32 gdmadst;				// GDMA destination address
 U32 gdmacnt;				// GDMA transfer count register  
 U32 gdmacon;				// GDMA control register 
 U8 * ptr;				// source buffer pointer 
 int i;


// Source address //////////////////////////////////////////////////////////////
 Print("\nInput source Address (0x%08x) : 0x",  (U32) GDMATestSrc);
 gdmasrc = get_number(16,0);
 if(gdmasrc == 0) 
  {
   gdmasrc = (U32) GDMATestSrc;  	// default source addr
  }

// Source size /////////////////////////////////////////////////////////////////

 if(get_last() != KEY_CR) { Print("\n"); }
 Print("\rInput Memory Test Size (0x%08x) 0x",  (U32) GDMATestSize);
 gdmacnt = get_number(16,0);
 if(gdmacnt == 0) 
  {
   gdmacnt = (U32) GDMATestSize;
  }

 if(
   gdmasrc == (U32) GDMATestSrc &&	//
   gdmacnt <=       GDMATestSize
  )
  {
   U8 ch;				// 
// Initialize source memory ////////////////////////////////////////////////////

   ch = 48;
   ptr = (U8 *) gdmasrc;
   for(i = 0; i < gdmacnt; ++i)
    {
     *ptr++ = ch++;
     if(ch >= 128)
      {
       ch = 48;
      } 
    }
  } 

// default set for memory to uart test /////////////////////////////////////////

 GdmaReset(gdma);  	// GDMA related register all clear to reset state

 gdmacon = 
  GDMACON_SI        |			// GDMA Stop Interrupt Enable 
  GDMACON_TD_MEMTOU |			// Transfer direction Memory to UART
  GDMACON_DST_FIX   |  			// destination address fix
  GDMACON_TW_BYTE; 			// 0 transfer width - byte

 if(uart) 
  {
   gdmacon |= GDMACON_MODE_U1;		// UART channel 1 
   uartcont  = UARTCONT1;		// save UARTCONT1 
  }
 else
  {
   gdmacon |= GDMACON_MODE_U0;		// UART channel 0
   uartcont  =  UARTCONT0;		// save UARTCONT0
  }

//
 gdmadst = 0;

// Configure GDMA channel //////////////////////////////////////////////////////

 uartcont &= ~UARTCON_TXM;
 if(gdma) 
  { 
   uartcont |= UARTCON_TXM_GDMA1REQ;
//
   GDMASRC1 = gdmasrc;			// GDMA 1 source address
   GDMADST1 = gdmadst;			// GDMA 1 destination address
   GDMACNT1 = gdmacnt;			// GDMA 1 transfer count	
   GDMACON1 = gdmacon;			// GDMA 1 control register 
  }
 else 
  {
   uartcont |= UARTCON_TXM_GDMA0REQ;
//
   GDMASRC0 = gdmasrc;			// GDMA 0 source address
   GDMADST0 = gdmadst;			// GDMA 0 destination address
   GDMACNT0 = gdmacnt;			// GDMA 0 transfer count 
   GDMACON0 = gdmacon;			// GDMA 0 control register
  }

 Print("\n");
 PrintGDMASRC(gdma);			// GDMA  source address
 PrintGDMADST(gdma);			// GDMA  destination address
 PrintGDMACNT(gdma);			// GDMA  transfer count	
 PrintGDMACON(gdma);			// GDMA  control register 
 Print("\n");

 if(uart == 1)
  {
   UARTCONT1 = uartcont;		// restore UARTCONT1 
  }
 else
  {
   UARTCONT0 = uartcont;		// restotr UARTCONT0
  } 


// Enable GDMA Channel Interrupt, Clear Done Flag and Global enable interrupt //  
 GdmaIntEnable(gdma);
 Enable_Int(nGLOBAL_INT); 
 
// Run GDMA Channel ////////////////////////////////////////////////////////////

 if(gdma) 
  {
   Gdma1DoneFlag = 0;
   GDMA1_RUN_ENABLE = 1;
   while(Gdma1DoneFlag == 0)  
    {					// wait GDMA1 
    }					// done interrupt 
  }
 else  
  {
   Gdma0DoneFlag = 0;
   GDMA0_RUN_ENABLE = 1;
   while(Gdma0DoneFlag == 0)  
    {					// wait GDMA0 
    } 					// done interrupt 
  }

 
 Disable_Int(nGLOBAL_INT); 
 GdmaIntDisable(gdma);

// restore UART configuration //////////////////////////////////////////////////

 uartcont = (uartcont & ~UARTCON_TXM) | UARTCON_TXM_INTREQ;

 if(uart == 1)
  {
   UARTCONT1 = uartcont;		// restore UARTCONT1 
  }
 else
  {
   UARTCONT0 = uartcont;		// restotr UARTCONT0
  } 

//           
 Print("\n");
}

////////////////////////////////////////////////////////////////////////////////
// GDMA STARTUP DIALOG /////////////////////////////////////////////////////////
////////////////////////////////////////////////////////////////////////////////


void GDMAStartUpDialog(void)
{
 U32 gdmacon;				// GDMA control register 
 U32 gdmasrc;				// GDMA source addreess register
 U32 gdmadst;				// GDMA destination register
 U32 gdmacnt;				// GDMA control register
 U32 uartcont;				// UART0/UART1 control register content
 U32 iopcon;				// IO Port control register 
 U8  inpsrc;				// source address input flag 
 U8  inpdst;				// destination address input flag
 TIME tm;				// transfer time 
 int gdma;				// GDMA channel 
 U32 mode;				// transfer mode 
 int it;				// selected menu item 

// 
 inpsrc = 1;				// input source address
 inpdst = 1;				// input destination address

// Initialize GDMA global registers/////////////////////////////////////////////
 gdmacon = 0;				// control register				
 gdmasrc = 0;				// source address register
 gdmadst = 0;				// destination address register
 gdmacnt = 0;				// counter register

/* STARTUP DIALOG FOR GDMA CONFIGURATION */ 

 Print("\n\nStartup dialog for GDMA configuration"); 

////////////////////////////////////////////////////////////////////////////////
// input GDMA channel //////////////////////////////////////////////////////////
////////////////////////////////////////////////////////////////////////////////

 while(1) 
  {
   Print("\nSelect GDMA channel                  [0/1/Q]");
   gdma = get_upper();			// read GDMA channel from console 
   if(gdma == 'Q')
    {
     return;
    }
   else
   if(gdma >= '0' && gdma <= '1')
    {
     gdma -= '0';			// to numerical value
     break;
    }
  }

////////////////////////////////////////////////////////////////////////////////
// input transfer mode /////////////////////////////////////////////////////////
////////////////////////////////////////////////////////////////////////////////

 while(1) 
  {
   Print("\nMode selection menu");
   Print("\n[0] SoftWare.");		// software (memory to memory)
   Print("\n[1] External EXTDREQ.");	// external request 
   Print("\n[2] UART0.");		// UART0 -> memory or memory -> UART0
   Print("\n[3] UART1.");		// UART1 -> memory ot memory -> UART1
   Print("\n[Q] Quit.");		//
   Print("\nSelect Item");
   it = get_upper();			// reag menu item from console
   if(it == 'Q')
    {
     return;
    }
   else  
   if(it >= '0' && it <= '3')
    {					// 
     break;
    } 
  } 

 switch(it) 
  {
   case '1' : 
// an external DMA request(nXDREQ) /////////////////////////////////////////////
    gdmacon |= GDMACON_MODE_EXTDREQ; 	// external DMA request 
    break; 

   case '2' : 
// the UART0 block /////////////////////////////////////////////////////////////
    gdmacon |= GDMACON_MODE_U0; 	// UART0 	
    break;      

   case '3' : 
// the UART1 block /////////////////////////////////////////////////////////////
    gdmacon |= GDMACON_MODE_U1; 	// UART1 	
    break;
   default  : 

// software (memory-to-memory) /////////////////////////////////////////////////
    gdmacon |= GDMACON_MODE_MEMMEM; 	// memory to memory
    break;  
  }

//
 mode = gdmacon & GDMACON_MODE;

////////////////////////////////////////////////////////////////////////////////
// GDMA_SI // [8] Stop interrupt enable (SI) ///////////////////////////////////       
////////////////////////////////////////////////////////////////////////////////

// Stop interrupt enable ///////////////////////////////////////////////////////

 while(1) 
  {
   Print("\nEnable stop interrupt                [Y/N/Q]");
   it = get_upper();
   if(it == 'Q')
    {
     return;
    }
   else  
   if(it == 'Y')
    {
     gdmacon |= GDMACON_SI;		// Stop Interrupt Enable 
     break;
    }
   else
   if(it == 'N')
    {
     break;
    }
  } 
        
////////////////////////////////////////////////////////////////////////////////
// GDMA_TD // [10] Transfer direction (for UART0/UART1 only) ///////////////////
////////////////////////////////////////////////////////////////////////////////

 if(
    (mode == GDMACON_MODE_U0) || 	// UART0
    (mode == GDMACON_MODE_U1)		// UART1 
  ) 
  {
   while(1) 
    {
     Print("\nTransfer Direction(UART0/UART1 only)");
     Print("\nMemory to [U]ART, UART to [M]emory   [U/M/Q]");
     it = get_upper();
     if(it == 'Q')
      {
       return;
      } 
     else
     if(it == 'U') 
      {
// GDMA_TD_MEMTOU // (memory to UART0/UART1) ///////////////////////////////////  

       gdmacon |= GDMACON_TD_MEMTOU;	// transfer direction = memory to UART
       gdmacon |= GDMACON_DST_FIX;	// fixed destination adress
       Print("\nDestination Addr Fixed.");
// 
       inpdst = 0;			// not input destination address
       break;
      }
     else 
     if(it == 'M') 
      {
// UART0/UART1 to memory ///////////////////////////////////////////////////////
       gdmacon |= GDMACON_TD_UTONEM;	// UART0/UART1 to memory
       gdmacon |= GDMACON_SRC_FIX;	// source address fix 
       Print("\nSource Addr Fixed.");

       inpsrc = 0;			// not input source adress 
       break;
      }
    } 
  }


 if(inpsrc == 1)
  {
////////////////////////////////////////////////////////////////////////////////
// Input Source Address and Source address increment or decrement //////////////
////////////////////////////////////////////////////////////////////////////////

   Print("\nTo select default source address 0x%08x\npress CR.", 
     (U32) GDMATestSrc
    );
   Print("\nInput source Address                    0x");
   gdmasrc = get_number(16,0);
   if(gdmasrc == 0) 
    {
     gdmasrc = (U32) GDMATestSrc;  	// default source address
    }

////////////////////////////////////////////////////////////////////////////////    
// Source address direction ////////////////////////////////////////////////////
////////////////////////////////////////////////////////////////////////////////    
   while(1)
    {
     Print("\nSource address decrement or increment[D/I/Q]");      
     it = get_upper();
     if(it == 'Q')
      {
       return;
      }
     else
     if(it == 'D')
      {
// source address decrement ////////////////////////////////////////////////////    
       gdmacon |= GDMACON_SRC_DEC;
       break;
      }
     else
     if(it == 'I')
      {
       break;
      }
    } 

// source address fix //////////////////////////////////////////////////////////    

   while(1)
    {
     Print("\nSource address fix                   [F/N/Q]");      
     it = get_upper();
     if(it == 'Q')
      {
       return;
      }
     else
     if(it == 'F')
      {
       gdmacon |= GDMACON_SRC_FIX;
       break;
      }
     else
     if(it == 'N')
      {
       break;
      }
    }
  }

 if(inpdst == 1)
  {
////////////////////////////////////////////////////////////////////////////////    
// Input Destination Address ///////////////////////////////////////////////////
////////////////////////////////////////////////////////////////////////////////    

   Print("\nTo select default dest. address  0x%08x\npress CR.", 
    (U32) GDMATestDst
    );

   Print("\nInput destination Address               0x");
   gdmadst = get_number(16,0);
   if(gdmadst == 0) 
    {
     gdmadst = (U32) GDMATestDst;  	// default destination address
    }

////////////////////////////////////////////////////////////////////////////////    
// Destination address direction ///////////////////////////////////////////////
////////////////////////////////////////////////////////////////////////////////    
   while(1) 
    {
     Print("\nDest. address decrement or increment [D/I/Q]");
     it = get_upper();
     if(it == 'Q')
      {
       return;
      }
     else
     if(it == 'D')
      {
// destination address decrement ///////////////////////////////////////////////
       gdmacon |= GDMACON_DST_DEC;
       break;
      }
     else
     if(it == 'I')
      {
       break;
      } 
    } 

// destination address fix /////////////////////////////////////////////////////

   while(1)
    {
     Print("\nDestination address fix              [F/N/Q]");
     it = get_upper();
     if(it == 'Q')
      {
       return;
      }
     else
     if(it == 'F')
      {
       gdmacon |= GDMACON_DST_FIX;
       break;
      }
     else
     if(it == 'N')
      {
       break;
      } 
    } 
  }

// Input Memory Size ///////////////////////////////////////////////////////////

 Print("\nInput transfer count                    0x");
 gdmacnt = get_number(16,0);
 if(gdmacnt < 1) 
  {
   gdmacnt = 1;
  }

////////////////////////////////////////////////////////////////////////////////
// [13:12] Transfer width (TW) /////////////////////////////////////////////////
////////////////////////////////////////////////////////////////////////////////

 while(1)
  {
   Print("\n\nTransfer width");
   Print("\n[B] Byte(8bit).");
   Print("\n[H] Halfword(16bit).");
   Print("\n[W] Word(32bit).");
   Print("\n[Q] Quit.");
   Print("Select Item");
   it = get_upper();
   if(it == 'B')
    {
// Transfer Width byte /////////////////////////////////////////////////////////
     gdmacon |= GDMACON_TW_BYTE; 	// byte	
     break;
    }
   else
   if(it == 'H')
    {
// Transfer Width = half word //////////////////////////////////////////////////
     gdmacon |= GDMACON_TW_HALFWORD; 	// half word 
     break;
    }    
   else
   if(it == 'W')
    {
// Transfer Width = word ///////////////////////////////////////////////////////
     gdmacon |= GDMACON_TW_WORD; 	// word 
     break;
    }
   else
   if(it == 'Q')
    {
     return;
    }
  }


////////////////////////////////////////////////////////////////////////////////
// GDMACON_SB // Single/Block mode (SB) ////////////////////////////////////////
////////////////////////////////////////////////////////////////////////////////


 while( mode == GDMACON_MODE_EXTDREQ )
  {
   Print("\nBlock mode or Single mode            [B/S/Q]");
   it = get_upper();
   if(it == 'Q')
    {
     return;
    }
   else
   if(it == 'B') 
    {
// GDMACON_SB_BLOCK // 1 = One nXDREQ initiates a whole DMA operation //////////
     gdmacon |= GDMACON_SB_BLOCK;
     break;
    }
   else
   if(it == 'S')
    {
// GDMACON_SB_SINGLE // 0 = One nXDREQ initiates a single DMA operation ////////
     gdmacon |= GDMACON_SB_SINGLE;
     break;
    }  
  } 

////////////////////////////////////////////////////////////////////////////////
// GDMACON_CM  // [14] Continuous mode (CN) ////////////////////////////////////
////////////////////////////////////////////////////////////////////////////////

 while( (mode != GDMACON_MODE_U0) && (mode != GDMACON_MODE_U1) ) 
  {
   Print("\nContinuous mode                      [Y/N]");
   it = get_upper();
   if(it == 'Q')
    {
     return;
    } 
   else
   if(it == 'Y') 
    {
// Continous mode ////////////////////////////////////////////////////////////// 
     if(mode != GDMACON_MODE_MEMMEM)
      {
/* NOTE: You can use Continuous mode together with a software */
/*       request mode. */
       Print("\nCautious. This mode can be used with");
       Print(" software request mode.");
      }  
     else
     if((gdmacon & GDMACON_SB) == GDMACON_SB_SINGLE)
      {
/*  NOTE: You should not use Block mode together with Demand mode, or */
/*        Single mode in conjunction with Continuous mode. */
       Print("\nCautious. Continuous mode should not");
       Print(" be used with single mode.");
      }
     else 
      {
       gdmacon |= GDMACON_CM;		
      } 
     break;
    }
   else
   if(it == 'N')
    {
     break;
    }    
  } 
           

////////////////////////////////////////////////////////////////////////////////
// GDMACON_DM // [15] Demand mode (DM) /////////////////////////////////////////
////////////////////////////////////////////////////////////////////////////////

 while( mode == GDMACON_MODE_EXTDREQ ) 
  {
   Print("\nDemand mode                          [Y/N/Q]");
   it = get_upper();
   if(it == 'Q')
    {
     return;
    } 
   else
   if(it == 'Y') 
    {
     if(
        ((gdmacon & GDMACON_SB) == GDMACON_SB_BLOCK) || // block mode 
/* NOTE: You should not use Block mode together with Demand mode, or */
/*       Single mode in conjunction with Continuous mode. */

         (gdmacon & GDMACON_CM)		// Continuos mode 	
/* NOTE: In Demand mode, you must clear the Single/Block */
/*       and Continuous mode control bits to 0. */
      ) 
      {
       Print("\nCautious. Block & Continuous mode must be zero.");  
      } 
     else
      {
       gdmacon |= GDMACON_DM;		// Demand Mode 
      }
//
     break;
    }
   else
   if(it == 'N')
    {
     break;
    }  
  } 
           

////////////////////////////////////////////////////////////////////////////////
// Configure GDMA channel //////////////////////////////////////////////////////
////////////////////////////////////////////////////////////////////////////////

 if(gdma == 1) 
  { 
   GDMASRC1 = gdmasrc;			// GDMA 1 source address
   GDMADST1 = gdmadst;			// GDMA 1 destination address
   GDMACNT1 = gdmacnt;			// GDMA 1 transfer count	
   GDMACON1 = gdmacon;			// GDMA 1 control register 
  }
 else 
 if(gdma == 0)
  {
   GDMASRC0 = gdmasrc;			// GDMA 0 source address
   GDMADST0 = gdmadst;			// GDMA 0 destination address
   GDMACNT0 = gdmacnt;			// GDMA 0 transfer count 
   GDMACON0 = gdmacon;			// GDMA 0 control register
  }

// 
 PrintGDMASRC(gdma);			// GDMA channel gdma source address
 PrintGDMADST(gdma);			// GDMA channel gdma destination address
 PrintGDMACNT(gdma);			// GDMA channel gdma transfer count	
 PrintGDMACON(gdma);			// GDMA channel gdma control register 
 Print("\n");

////////////////////////////////////////////////////////////////////////////////
// configure UART0/UART1 in UART modes /////////////////////////////////////////
////////////////////////////////////////////////////////////////////////////////

 if(mode == GDMACON_MODE_U0)
  {
////////////////////////////////////////////////////////////////////////////////
// UART0 ///////////////////////////////////////////////////////////////////////
////////////////////////////////////////////////////////////////////////////////

   uartcont  =  UARTCONT0;		// save UARTCONT0

   if((gdmacon & GDMACON_TD) == GDMACON_TD_MEMTOU)
    {
// memory => UART0 ///////////////////////////////////////////////////////////// 
     uartcont &= ~UARTCON_TXM;		// clear UART0 Tx mode bits
     uartcont |= (gdma ? UARTCON_TXM_GDMA1REQ : UARTCON_TXM_GDMA0REQ);
    }
   else
    {
// UART0 to memoty /////////////////////////////////////////////////////////////
     uartcont &= ~UARTCON_RXM;		// clear UART0 Rx mode bits
     uartcont |= (gdma ? UARTCON_RXM_GDMA1REQ : UARTCON_RXM_GDMA0REQ);
    }
  }
 else
 if(mode == GDMACON_MODE_U1)
  {
////////////////////////////////////////////////////////////////////////////////
// configure UART1 /////////////////////////////////////////////////////////////
////////////////////////////////////////////////////////////////////////////////

   uartcont  =  UARTCONT1;		// save UARTCONT1 

   if((gdmacon & GDMACON_TD) == GDMACON_TD_MEMTOU)
    {
// memory to UART1 /////////////////////////////////////////////////////////////
     uartcont &= ~UARTCON_TXM;		// clear UART1 Tx mode bits 
     uartcont |= (gdma ? UARTCON_TXM_GDMA1REQ : UARTCON_TXM_GDMA0REQ);
    }
   else
    {
// UART1 to memory /////////////////////////////////////////////////////////////
     uartcont &= ~UARTCON_RXM;		// clear UART1 Tx mode bits 
     uartcont |= (gdma ? UARTCON_RXM_GDMA1REQ : UARTCON_RXM_GDMA0REQ);
    }
  }
 else
 if(mode == GDMACON_MODE_EXTDREQ)
  {
   U32 dreq_hight; 			// DREQ Active Hight mask 
   U32 dreq_filter_on;		 	// DREQ Filtering on mask
   U32 dreq_en;		 		// DREQ Enable bit mask
   U32 dack_hight; 			// DACK Active Hight mask
   U32 dack_en; 			// DACK Enable/Disable bit mask
   U32 iop_con;
  
//  
   dreq_hight     = ((gdma) ? IOPCON_DREQ1_HIGHT     : IOPCON_DREQ0_HIGHT);
   dreq_filter_on = ((gdma) ? IOPCON_DREQ1_FILTER_ON : IOPCON_DREQ0_FILTER_ON);
   dreq_en        = ((gdma) ? IOPCON_DREQ1_EN        : IOPCON_DREQ0_EN);
   dack_hight     = ((gdma) ? IOPCON_DACK1_HIGHT     : IOPCON_DACK0_HIGHT);
   dack_en        = ((gdma) ? IOPCON_DACK1_EN        : IOPCON_DACK0_EN);

// 		
   iopcon = iop_con = IOPCON;		// save IOPCON register 

   while(1)
    {    
     Print("\nDREQ%d & DACK%d configuration. IOPCON register.", gdma, gdma);

     Print("\n[H] DREQ%d Active Hight       %s", gdma, 
      (iop_con & dreq_hight) ?    	
       "1(active hight)" :		
       "0(active low)"
     ); 
     Print("\n[F] DREQ%d Filtering on       %s", gdma, 
      (iop_con & dreq_filter_on) ?	
       "1(on)"		:
       "0(off)"
     );

     Print("\n[E] DREQ%d Enable bit         %s", gdma, 
      (iop_con & dreq_en) ?       	
      "1(enable)"	:
      "0(disable)"
     ); 

/* Control external DMA acknowledge 0/1 output for port 14/15 (DACK0/DACK1) */
     Print("\n[A] DACK%d Active Hight       %s", gdma, 
      (iop_con & dack_hight) ?
      "1(active hight)"	:
      "0(active low)"	
     ); 

     Print("\n[N] DACK%d Enable/Disable bit %s", gdma, 
      (iop_con & dack_en) ?
      "1(enable)"	:
      "0(disable)"
     ); 

     Print("\n[W] Write IOPCON");
     Print("\n[Q] Quit");
     Print("\n       Select Item");
     if(it == 'Q')
      {
       return;
      } 
     
     if(it == 'W')
      {
       IOPCON = iop_con;
       break;
      }

     switch(iop_con)
      {
       case 'H' : iop_con ^= dreq_hight;     break; // DREQ Active Hight
       case 'F' : iop_con ^= dreq_filter_on; break; // DREQ Filtering on	
       case 'E' : iop_con ^= dreq_en;        break; // DREQ Enable bit
       case 'A' : iop_con ^= dack_hight;     break; // DACK Active Hight
       case 'N' : iop_con ^= dack_en;        break; // DACK Enable/Disable bit
       default  : break; 
      }     
    }
  }

// 
 Print("\nStart GDMA operation                 [Y/N]"); 
 it = get_upper();
 if(it != 'Y')
  {
   return;
  } 

//
 if(mode == GDMACON_MODE_U1)
  {
   UARTCONT1 = uartcont;
  }
 else
 if(mode == GDMACON_MODE_U0)
  {
   UARTCONT0 = uartcont;
  } 

//
 TimerReset(1);
 TimerInit(1,(ONE_SECOND/TICKS_PER_SECOND));
 TimerStart(1);

// Enable GDMA Channel Interrupt, Clear Done Flag and Global enable interrupt //  
 GdmaIntEnable(gdma);			// 
 Enable_Int(nGLOBAL_INT);

// Run GDMA Channel ////////////////////////////////////////////////////////////

 if(gdma) 
  {
   Gdma1DoneFlag = 0;
   GDMA1_RUN_ENABLE = 1;		// run GDMA1  
  }
 else  
  {
   Gdma0DoneFlag = 0;
   GDMA0_RUN_ENABLE = 1;		// run GDMA0
  }

// 
 if(gdmacon & GDMACON_SI)
  {
// Stop Interrupt Enable => wait done interrupt //////////////////////////////// 

   if(gdma) 
    {
     while(Gdma1DoneFlag == 0)  
      {					// wait GDMA1 
      }					// done interrupt 
    }
   else 
    {
     while(Gdma0DoneFlag == 0)  
      {					// wait GDMA0 
      } 				// done interrupt 
    }
  }
 else
  {
// Stop Interrupt disabled => wait busy bit clearing /////////////////////////// 
   if(gdma == 1)
    {
     while((GDMACON1 & GDMACON_BS) != 0)
      {
      } 
    }
   else
   if(gdma == 0)
    {
     while((GDMACON0 & GDMACON_BS) != 0)
      {
      } 
    }
  } 

 Disable_Int(nGLOBAL_INT); 
 GdmaIntDisable(gdma);

////////////////////////////////////////////////////////////////////////////////
// restore UART configuration //////////////////////////////////////////////////
////////////////////////////////////////////////////////////////////////////////

 uartcont &= ~(UARTCON_TXM | UARTCON_RXM);
 uartcont |= UARTCON_TXM_INTREQ | UARTCON_RXM_INTREQ; 

 if(mode == GDMACON_MODE_U1)
  {
   UARTCONT1 = uartcont;
  }
 else
 if(mode == GDMACON_MODE_U0)
  {
   UARTCONT0 = uartcont;
  } 
 else
 if(mode == GDMACON_MODE_EXTDREQ)
  {
   IOPCON = iopcon; 			// restore IOPCON register 
  }

//           
 TimerStop(1);
 Print("\n");

 GetSysTime(1,& tm); 
 TimerReset(1);
 Print
  (
   "GDMA Transfer time %d:%d:%d.%d",
    tm.tm_hour	,
    tm.tm_min	,
    tm.tm_sec	,
    tm.tm_ms
  );
}


////////////////////////////////////////////////////////////////////////////////
// GDMA TEST TOP MEMU SELECT FUNCTION //////////////////////////////////////////
////////////////////////////////////////////////////////////////////////////////

void GdmaTest(void)
{
 char item;

 SysSetInterrupt(nGDMA0_INT , GDMA0isr);
 SysSetInterrupt(nGDMA1_INT , GDMA1isr);

 do 
  {
   Print("\n\nGDMA TEST");
   Print("\n[0] GDMA0 configuration view.");
   Print("\n[1] GDMA1 configuration view.");
   Print("\n[T] GDMA Memory to Memory test.");
   Print("\n[U] GDMA UART0/UART1 to Memory transfer test.");
   Print("\n[M] GDMA Memory to UART0/UART1 transfer test.");
   Print("\n[C] GDMA start up dialog for setup DMA mode.");
   Print("\n[Q] QUIT - Return to main menu.");
   Print("\nSelect Item");
   item = get_upper();

   switch(item) 
    {
     case '0': GDMAConfigView(0); break;	// view GDM0 configuration
     case '1': GDMAConfigView(1); break;	// view GDM1 configuration
      
     case 'C': 
// GDMA start up dialog for setup DMA mode /////////////////////////////////////
      GDMAStartUpDialog(); 	   		// GDMA Configure dialog
      break;

     case 'T': 
// GDMA Memory to Memory test //////////////////////////////////////////////////
      GdmaMemMemTest();    			// Memory to memory transfer 
      break;

     case 'U':
// UART0/UART1 to memory test ////////////////////////////////////////////////// 
      UartToMemTest();	   		  	// UART0/UART1 to memory test
      break;

     case 'M':
// GDMA Memory to UART0/UART1 transfer test //////////////////////////////////// 
      MemToUartTest();	   		  	// UART0/UART1 to memory test
      break;

     case 'Q':
// QUIT - Return to main menu //////////////////////////////////////////////////
      return; 

     default : 
      break;
    }

   Print("\nPress any key to continue.\n");
   get_byte();
  } 
 while(item != 'Q');
}


/////////////////////////////////////////////////////////////////////////////////
// copy function, using GDMA0 interrupt /////////////////////////////////////////
/////////////////////////////////////////////////////////////////////////////////

//
// dmadst = destination address
// dmasrc = source address
// tcnt   = transfer count (transfer size = tcnt * (2**width) )
// Width  = bus width (0 = byte, 1 = half word, 2 = word)
//

void gdma0copy(U32 dmadst, U32 dmasrc,int tcnt, int Width)
{
 U32 DMA_CON_SET;			// control register content

// 
 Disable_Int(nGDMA0_INT); 		// Disable GDMA 0 interupt
 SysSetInterrupt(nGDMA0_INT, GDMA0isr); // Vector setup
 Gdma0DoneFlag = 0;       		// clear GDMA0 transmit done flag

// 
 GDMASRC0 = dmasrc;			// GDMA0 source address 
 GDMADST0 = dmadst;			// GDMA0 destination address 
 GDMACNT0 = tcnt;			// transfer count

 DMA_CON_SET = GDMACON_MODE_MEMMEM;
	
 switch(Width) 
  {
   case 0 : 
// transfer width = byte ///////////////////////////////////////////////////////
    DMA_CON_SET |= GDMACON_TW_BYTE; 
    break;        

   case 1 : 
// transfer width = half word //////////////////////////////////////////////////
    DMA_CON_SET |= GDMACON_TW_HALFWORD; 
    break;

   case 2 : 
   default: 
// transfer width = word ///////////////////////////////////////////////////////
    DMA_CON_SET |= GDMACON_TW_WORD; 
    break;
  }

 Enable_Int(nGDMA0_INT);    		// Enable GDMA 0 interrupt
 Enable_Int(nGLOBAL_INT);		// Enable global interrupt
 GDMACON0 = 
  DMA_CON_SET | 
  GDMACON_RE;  		   		// Run Enable

// 
 while(!Gdma0DoneFlag);			// wait GDMA 0 Done Interrupt 
}

/////////////////////////////////////////////////////////////////////////////////
// Copy function, using GDMA1 ///////////////////////////////////////////////////
/////////////////////////////////////////////////////////////////////////////////

void gdma1copy(U32 dmadst, U32 dmasrc,int tcnt, int Width)
{
 long DMA_CON_SET;

// 
 Disable_Int(nGDMA1_INT); 		// Disable GDMA 1 interupt
 SysSetInterrupt(nGDMA1_INT, GDMA1isr); // Set GDMA channel 1 ISR
 Gdma1DoneFlag = 0;       		// clear GDMA1 done flag

// configure GDMA channel 1 registers //////////////////////////////////////////

 GDMASRC1 = dmasrc;			// source address register 
 GDMADST1 = dmadst;			// destination address register 
 GDMACNT1 = tcnt;			// transfer count 

// GDMA 1 Control register /////////////////////////////////////////////////////
 DMA_CON_SET = GDMACON_MODE_MEMMEM;	// mode = memory to memory transfer 
	
 switch(Width) 
  {
   case 0 : 
// Transfer Width byte /////////////////////////////////////////////////////////
    DMA_CON_SET |= GDMACON_TW_BYTE; 	// 
    break;        

   case 1 : 
// Transfer Width = half word //////////////////////////////////////////////////
    DMA_CON_SET |= GDMACON_TW_HALFWORD; 	// 
    break;

   case 2 : 
   default: 
// Transfer Width = word ///////////////////////////////////////////////////////
    DMA_CON_SET |= GDMACON_TW_WORD; 	//
    break;
  }

// 
 Enable_Int(nGDMA1_INT);    		// Enable GDMA 0 interrupt
 Enable_Int(nGLOBAL_INT);		// Enable global Int 
 GDMACON1 = 				// Set GDMA 1 control register
  DMA_CON_SET | 
  GDMACON_RE;				// Run Enable 

 while(!Gdma1DoneFlag);
}

/////////////////////////////////////////////////////////////////////////////////
// DMA auto test ////////////////////////////////////////////////////////////////
/////////////////////////////////////////////////////////////////////////////////

// src   = source address
// dst   = destination address
// tsize = size in words
//  

int DmaAutoTest(U32 src, U32 dst, int tsize,int lsize)
{
 U32 * ptr;
 int cnt;
 int pat;
 int i;

// For use non-cacheable area
 dst |= 0x4000000;
 src |= 0x4000000;

// Word Test

 ptr = (U32 *) src;			/* source address */
 cnt = tsize;				/* source size in words */ 
 pat = 0;				/* start parameter for PatternGen */

 while(cnt--) 
  {
   *ptr++ = (U32) PatternGen(pat++);
  }


 Print("\n    DMA Channel 0 Test (Long) ....    ");

 for(i = 0; i < lsize ; i++) 
  {
   gdma0copy(src, dst, tsize, 2) ;
   if( !wcomp((U32 *)src, (U32 *)dst, tsize, 0) )
    { 
     Print("Fail.") ;
     return(0);
    }
   PrintRotSlash(i%4) ;
  }
 Print("Ok.") ;

// 
 ptr = (U32 *) dst;
 cnt = tsize;
 while(cnt--) { *ptr++ = 0; } 

// Half Word Test
 Print("\n    DMA Channel 0 Test (Short) ...    ") ;
 for( i = 0; i < lsize ; ++i) 
  {
   gdma0copy(src, dst, tsize * 2, 1) ;
   if( !scomp((U16 *)src, (U16 *)dst, tsize, 0) )
    { 
     Print("Fail.") ;
     return(0);
    }
   PrintRotSlash(i%4) ;
  }
 Print("Ok.");

// 
 ptr = (U32 *) dst;
 cnt = tsize;
 while(cnt--) { *ptr++ = 0; } 

// Byte Test
 Print("\n    DMA Channel 0 Test (Byte) ....    ");
 for (i=0 ; i < lsize ; i++) 
  {
   gdma0copy(src, dst, tsize * 4, 0);
   if( !bcomp((U8 *)src, (U8 *)dst, tsize, 0) )
    { 
     Print("Fail.");
     return(0);
    }
   PrintRotSlash(i%4);
  }
 Print("Ok.");

// 
 ptr = (U32 *) dst;
 cnt = tsize;
 while(cnt--) { *ptr++ = 0; } 

// Word Test
 Print("\n    DMA Channel 1 Test (Long) ....    ");
 for (i = 0; i < lsize; i++) 
  {
   gdma1copy(src, dst, tsize, 2);
   if( !wcomp((U32 *)src, (U32 *)dst, tsize, 0) )
    { 
     Print("Fail.");
     return(0);
    }
   PrintRotSlash(i%4);
  }
 Print("Ok.");

// 
 ptr = (U32 *) dst;
 cnt = tsize;
 while(cnt--) { *ptr++ = 0; } 

// Half Word Test
 Print("\n    DMA Channel 1 Test (Short) ...    ") ;
 for (i = 0; i < lsize ; i++) 
  {
   gdma1copy(src, dst, tsize * 2, 1);
   if ( !scomp((U16 *)src, (U16 *)dst, tsize, 0) )
    { 
     Print("Fail.");
     return 0;
    }
   PrintRotSlash(i%4);
  }
 Print("Ok.");

// 
 ptr = (U32 *) dst;
 cnt = tsize;
 while(cnt--) { *ptr++ = 0; } 

// Byte Test
 Print("\n    DMA Channel 1 Test (Byte) ....    ") ;
 for (i=0 ; i < lsize ; i++) 
  {
   gdma1copy(src, dst, tsize * 4, 0);
   if ( !bcomp((U8 *)src, (U8 *)dst, tsize, 0) )
    { 
     Print("Fail.") ;
     return(0);
    }
   PrintRotSlash(i%4) ;
  }
 Print("Ok") ;
 return(1);
}
