/*************************************************************************/
/*                                                                       */
/* FILE NAME                                            VERSION          */
/*                                                                       */
/*      init.s                              SNDS100 Board version 1.0    */
/*                                                                       */
/* COMPONENT                                                             */
/*                                                                       */
/*                                                                       */
/* DESCRIPTION                                                           */
/*                                                                       */
/*      This file contains the target processor dependent initialization */
/*      routines (boot code) and data.                                   */
/*                                                                       */
/* AUTHOR                                                                */
/*                                                                       */
/*                                                                       */
/* DATA STRUCTURES                                                       */
/*                                                                       */
/*                                                                       */
/* FUNCTIONS                                                             */
/*                                                                       */
/*                                                                       */
/* DEPENDENCIES                                                          */
/*                                                                       */
/*      snds.a                              System constants             */
/*                                                                       */
/* HISTORY                                                               */
/*                                                                       */
/*         NAME            DATE                    REMARKS               */
/*                                                                       */
/*        hbahn         01-25-1998      Created initial version 1.0      */
/*                                                                       */
/*************************************************************************/

/* 			*/
/* Modified by 		*/
/* Dmitriy Cherkashin 	*/
/* dch@ucrouter.ru	*/
/* 2002,2003,2004	*/
/*			*/

#include "init_gnu.h"

.globl __main
__main:
/* --- Setup interrupt / exception vectors */
#ifdef ROM_AT_ADDRESS_ZERO
/* If the ROM is at address 0 this is just a sequence of branches */
        B       Reset_Handler			/* Reset (mode = Supervisor) */
        B       Undefined_Handler		/* Udefined instruction (mode = Undefined) */
        B       SWI_Handler			/* Software interrupt (mode = Supervisor) */
        B       Prefetch_Handler		/* Abort prefetch (mode = Abort) */
        B       Abort_Handler			/* Abort data (mode = Abort) */
        NOP     				/* Reserved vector (mode = Reserved) */
/* NOP = Pseudo Instruction, translated MOV r0,r0  */
        B       IRQ_Handler			/* IRQ (mode = IRQ) */
        B       FIQ_Handler			/* FIQ (mode = FIQ) */

/************************************************************************/
/* Action on Entering an Exception					*/
/* When handling an exception, the ARM7TDMI:				*/
/* 1. Preserves the address of the next instruction in the 		*/
/* appropriate Link Register. If the exception has been entered from 	*/
/* ARM state, then the address of the next instruction is copied into 	*/
/* the Link Register (that is, current PC + 4 or PC + 8 depending on 	*/
/* the exception. If the exception has been entered from THUMB state,	*/
/* then the value written into the Link Register is the current PC 	*/
/* offset by a value such that the program resumes from the correct 	*/
/* place on return from the exception.					*/	
/* 2. Copies the CPSR into the appropriate SPSR				*/
/* 3. Forces the CPSR mode bits to a value which depends on the 	*/
/*    exception								*/
/* 4. Forces the PC to fetch the next instruction from the relevant 	*/
/*    exception vector							*/
/************************************************************************/

#else 
/* Otherwise we copy a sequence of LDR PC instructions over the vectors */
/* (Note: We copy LDR PC instructions because branch instructions */
/* could not simply be copied, the offset in the branch instruction */
/* would have to be modified so that it branched into ROM. Also, a */
/* branch instructions might not reach if the ROM is at an address */
/* > 32M). */
        MOV     R8, #0
        ADR     R9, Vector_Init_Block
        LDMIA   R9!, {R0-R7}
        STMIA   R8!, {R0-R7}
        LDMIA   R9!, {R0-R7}
        STMIA   R8!, {R0-R7}

/* Now fall into the LDR PC, Reset_Addr instruction which will continue */
/* execution at 'Reset_Handler' */

Vector_Init_Block:
        LDR     PC, Reset_Addr
        LDR     PC, Undefined_Addr
        LDR     PC, SWI_Addr
        LDR     PC, Prefetch_Addr
        LDR     PC, Abort_Addr
        NOP
        LDR     PC, IRQ_Addr
        LDR     PC, FIQ_Addr

Reset_Addr:     .word Reset_Handler
Undefined_Addr: .word Undefined_Handler
SWI_Addr:       .word SWI_Handler
Prefetch_Addr:  .word Prefetch_Handler
Abort_Addr:     .word Abort_Handler
                .word 0       	/* Reserved vector */
IRQ_Addr:       .word IRQ_Handler
FIQ_Addr:       .word FIQ_Handler
#endif 						/* ROM_AT_ADDRESS_ZERO */

/*==========================================================*/
/* The Default Exception Handler Vector Entry Pointer Setup */
/*==========================================================*/

/* Exeption Priorities */
/* 1. Reset (Highest priority) */
/* 2. Data Abort */
/* 3. FIQ */
/* 4. IRQ */
/* 5. Prefetch Abort */
/* 6. Undefined Instruction. Software interrupt (Lowest priority) */

/************************************************************************/
/* FIQ Handler 								*/
/************************************************************************/

FIQ_Handler:
/************************************************************************/
/* The FIQ exeption is generated externally by asserting the FIQ	*/
/* input on the processor. FIQ is designated to support a data 		*/
/* transfer  channel process, and has sufficient private registers 	*/
/* to remove the need for registe saving in sutch applications,		*/
/* therefore minimizing the overhead of context switching.		*/
/*									*/
/* Fast interrupts are disabled when the F bit in the CPSR is set. 	*/
/* If the F bit is clear, ARM checks for an FIQ an instruction		*/
/* boundaries.								*/
/*									*/
/* Then an FIQ is detected, the following actions are performed:	*/
/*									*/
/* F14_fiq = address of next instruction to be executed + 4		*/
/* SPSR_fiq = CPSR							*/
/* CPSR[4:0] = 0b10001		// Enetr FIQ mode			*/
/* CPSR[5] = 0 		// Execute in ARM state				*/
/* CPSR[6] = 1			// Disable fast interrupts 		*/
/* CPSR[7] = 1			// Disable normal interrupts		*/
/* PC = 0x0000001C 							*/
/*									*/
/* To return after servicing the interrupt, use:			*/
/*									*/
/* SUBS PC,R14,#4							*/
/*									*/
/* This restores both the PC (from R14_fiq) and CPSR (from SPSR_fiq), 	*/
/* and resumes execution of the interrupted code.			*/
/*									*/	
/* The FIQ vector is deliberately the last vector to allow the FIQ 	*/
/* exeption-handler software to be placed directly at address 		*/
/* 0x0000001C, without requiring a branch instruction from the vector.  */	
/************************************************************************/
	STMFD	sp!, {r0-r7, lr}
/* out to UART0 character F */
/*	LDR	r1, =UARTTXH0 */
/*	LDR	r0, =0x46 */	/* 0x46 = 'F' */
/*	STR	r0, [r1] */
#ifdef LED_ONLY
#else 	
	BL	ISR_FiqHandler
#endif 								/* LED_ONLY */	
	LDMFD	sp!, {r0-r7, lr}
	SUBS	pc, lr, #4

/************************************************************************/
/* LRM, STM suffixes :							*/
/* IA - Increment after  = EA - Empty ascending stack			*/
/* IB - Increment before = FA - Full ascending stack			*/
/* DA - Decrement after  = ED - Empty descending stack			*/
/* DB - Decrement before = FD - Full descending stack			*/
/* Full stack  - the stack pointer point to the last item in the stack	*/
/* Empty stack - the stack pointer point to the next item in the stack	*/
/* ARM The ARM-Thumb Procedure Call Standard (ATPCS),and ARM and Thumb  */ 
/* C and C++ compilers always use a full descending stack.		*/
/************************************************************************/


/************************************************************************/
/* IRQ Handler 								*/  
/************************************************************************/
IRQ_Handler:
/************************************************************************/
/* The IRQ exeption is generated externally by asserting the IRQ	*/
/* input on the processor. It has a lower priority than FIQ, and	*/
/* is masked out when an FIQ sequence is entered.			*/
/*									*/
/* Interrupts are disabled when the I bit in CPSR is set. If the I 	*/
/* bit is clear, ARM checks for an IRQ at insruction boundaries.	*/
/* The I bit can only be changed from a privileged mode.		*/
/*									*/
/* Then an IRQ is detected, the following actions are performed:	*/
/*									*/
/* R14_irq = adress of next instruction to be executed + 4		*/
/* SPSR_irq = CPSR							*/
/* CPSR[4:0] = 0b10010		// Enter IRQ mode			*/	
/* CPSR[5] = 0			// Execute in ARM state			*/
/* CPSR[6] = unchanged		// FIQ mask bit (1 - disable)		*/
/* CPSR[7] = 1			// Disable normal interrupts		*/
/* PC = 0x00000018							*/
/*									*/
/* To return after servicing the interrupt, use:			*/
/*									*/
/* SUBS PC,R14,#4							*/
/*									*/
/* This restore both PC (from R14_irq) and CPSR (from SPSR_irq),	*/
/* and resumes execution of the interrupted code.			*/
/************************************************************************/
	STMFD	sp!, {r0-r12, lr}

/* out to UART0 character I */
/*	LDR	r1, =UARTTXH0   */		
/*	LDR	r0, =0x49	*/	 /* 0x49 = 'I' */
/*	STR	r0, [r1]	*/	

#ifdef LED_ONLY
#else 	
	BL	ISR_IrqHandler
#endif 							/* LED_ONLY */
	LDMFD	sp!, {r0-r12, lr}
	SUBS	pc, lr, #4

/************************************************************************/
/* Prefetch Handler 							*/
/************************************************************************/
Prefetch_Handler:
/************************************************************************/
/* Prefetch Abort (instruction fetch memory abort)			*/
/* F14_abt = address of the aborted instruction + 4			*/
/* SPSR_abt = CPSR							*/	
/* CPSR[4:0] = 0b10111		// enter Abort mode 			*/
/* CPSR[5] = 0			// Execute in ARM state			*/
/* CPSR[6] = unchanged		// FIQ mask bit				*/
/* CPSR[7] = 1			// Disable normal interrupts		*/
/* PC = 0x0000000C							*/
/*									*/
/* To return after fixing the reason for the abort, use:		*/
/*									*/
/* SUBS PC,R14,#4							*/
/*									*/
/* This restores both PC (from R14_abt) and CPSR (from SPSR_abt),	*/
/* and return to the aborted instruction. 				*/
/************************************************************************/
	STMFD	sp!, {r0-r12, lr}

/* out to UART0 character P*/
	LDR	r1, =UARTTXH0
	LDR	r0, =0x50		/* 0x50 = 'P' */
	STR	r0, [r1]
#ifdef LED_ONLY
#else	
        SUB     r0,lr,#4		/* r0 = address of abborted instruction */
	BL	ISR_PrefetchHandler	/* Branch and link System Prefetch Abort Handler */
/* ISR_PrefetchHandler defined in isr.c as: */
/* void ISR_PrefetchHandler(void *adr), where */
/*      adr = address of abborted instruction */
#endif 							/* LED_ONLY */
	LDMFD	sp!, {r0-r12, lr}
	SUBS	pc, lr, #4

/************************************************************************/
/* Abort Handler 							*/
/************************************************************************/
Abort_Handler:
/************************************************************************/
/* A Data Abort exception occures before any following instructions or	*/
/* exeptions have altered the state of the CPU.				*/
/*									*/
/* F14_abt = address of the aborted instruction + 8			*/
/* SPSR_abt = CPSR							*/
/* CPSR[4:0] = 0b10111			// Enter Abort mode		*/
/* CPSR[5] = 0				// Execite in ARM state		*/
/* CPSR[6] = unchanged							*/
/* CPSR[7] = 1				// Disable normal interrupts	*/
/* PC = 0x00000010							*/
/*									*/
/* To return after fixing the reason for the abort use:			*/
/* 									*/
/*  SUBS PC,R14,#8							*/
/*  									*/
/* This restores both the PC (from R14_abt) and CPSR (from SPSR_abt),	*/	
/* and returns to re-execute the aborted instruction.			*/
/************************************************************************/
	STMFD	sp!, {r0-r12, lr}
/* out to UART0 character 'D'  */
	LDR	r1, =UARTTXH0		
	LDR	r0, =0x44		/* 0x44 = 'D' */ 
	STR	r0, [r1]		

#ifdef LED_ONLY
#else 
        SUB     r0,lr,#4		/* r0 = address of abborted data instruction */
	BL	ISR_AbortHandler	/* undefined handler */
/* ISR_AbortHandler defined in isr.c as: */
/* void ISR_AbortHandler(void *adr) */
#endif 
	LDMFD	sp!, {r0-r12, lr}
	SUBS	pc, lr, #8

/************************************************************************/
/* undefined Handler 							*/
/************************************************************************/
Undefined_Handler:
/************************************************************************/
/*Action on Entering an Exeption. Then handling an exeption, 		*/
/*the ARM7TDMI:								*/
/*1. Preserves the address of the next instruction in the 		*/
/*appropriate Link register. In ARM state address of next instruction	*/
/*(PC+4 or PC=8) is copied into the link register.			*/
/*2. Copies the CPSR in the appropriate SPSR				*/
/*3. Forces the CPSR mode bits to value which depends on exeption	*/
/*4. Forces the PC to fetch the next instruction from the relevant 	*/
/* exeption vector 							*/
/************************************************************************/
/* 1. Then the Undefined Instruction occurs, the following 		*/
/* actions are performed: 						*/
/* 									*/
/* R14_und = address of next instruction after the  			*/
/* undefined instruction 						*/ 
/* SPSR_und = CPSR 							*/
/* CPSR[4:0] = 0b11011 	// enter Undefined mode 			*/
/* CPSR[5] = 0		// execute ARM state 				*/
/* CPSR[6] is unchanged 						*/
/* CPSR[7] = 1								*/
/* PC = 0x00000004   							*/
/* 									*/
/* To return after emulating the undefined instruction use: 		*/
/* MOVS PC,R14 								*/
/* 									*/
/* This restores the PC (from R14_und) and CPSR (from SPSR_und) and	*/
/* return to the instruction following the undefined instruction. 	*/
/* 									*/
/* 2. MOVS Description  						*/
/* Then Rd (destination register) is R15 and S flag is set the 		*/
/* result of the operation is placed in R15 and the SPSR corresponding  */
/* to the current mode is moved to CPSR. This allows state changes which*/
/* atomaticaly restore both PC and CPSR 				*/
/* 									*/ 
/* 3. C code for ISR_UndefHandler :  					*/
/* 									*/
/* void ISR_UndefHandler(void *adr) 					*/
/* {									*/
/*  Print("\rUndefined Address : %08x ",adr); 				*/
/*  Print("\rUndefined Data : %08x ",*adr); 				*/
/* } 									*/
/* 									*/
/* ASM code for ISR_UndefHandler: 					*/
/* STMFD    r13!,{r4,r14} 						*/
/* MOV      r4,r0 							*/
/* MOV      r1,r4 							*/
/* ADD      r0,pc,#0x14 ; #0x58 					*/
/* BL       Print 							*/
/* LDR      r0,[r4,#0] 							*/
/* MOV      r1,r0 							*/
/* ADD      r0,pc,#0x20 ; #0x74 					*/
/* BL       Print 							*/
/* LDMFD    r13!,{r4,pc}  						*/
/* 									*/
/* r0  = undefined (not address of undefined instruction),  		*/
/* r14 = address of next instruction after undefined 			*/
/* LDMFD    r13!,{r4,pc} return control to next instructon after 	*/
/* udefined witout mode change (mode remain undefined mode). 		*/
/************************************************************************/

	STMFD	sp!, {r0-r12,lr}	/* save [r0-r12,r14] */
/* Out to UART0 character */
	LDR	r1, =UARTTXH0		 
	LDR	r0, =0x55		/* 0x54 = 'U' */
	STR	r0, [r1]

        SUB     r0,lr,#4		/* r0 = address undefined instruction */
#ifdef LED_ONLY
#else 
	BL	ISR_UndefHandler	/* conditional branch */
#endif 
	LDMFD	sp!, {r0-r12, pc}^	/* restore r0-r12, CPSR, PC = next instruction */

SWI_Handler:
/************************************************************************/
/* Software Interrupt exception 					*/
/* The software Interrupt instruction (SWI) enters Supervisor mode to 	*/
/* request a particular supervisor (operating system) function. When 	*/
/* a SWI is executed, the following actions are performed:		*/
/*									*/
/* R14_svc = address of next instruction after the SWI instruction	*/
/* SPSR_svc = CPSR							*/
/* CPSR[4:0] = 0b10011		// Enter Supervisor mode 		*/
/* CPSR[5] = 0 		// Execute in ARM state 			*/
/* CPSR[6] = unchanged							*/
/* CPSR[7] = 1			// Disable normal interrupt		*/
/* PC = 0x00000008							*/
/************************************************************************/
	STMFD	sp!, {r0-r12,lr}
/* out to UART0 character S */
	LDR	r1, =UARTTXH0	
	LDR	r0, =0x53		/* 0x53 = 'S' */
	STR	r0, [r1]	

	LDR	r0, [lr, #-4]
	BIC	r0, r0, #0xff000000
	CMP	r0, #0xff
	BEQ	MakeSVC
#ifdef  LED_ONLY
#else 
        BL      ISR_SwiHandler
#endif 
/* Note : r0 contain comment field (24 bits in ARM mode), so SWI handler */
/* may be defined as : void ISR_SwiHandler(int cmd) */
	LDMFD	sp!, {r0-r12, pc}^
MakeSVC:
	MRS	r1, spsr
	BIC	r1, r1, #MODE_MASK
	ORR	r2, r1, #SUP_MODE
	MSR	spsr_csxf, r2	
	LDMFD	sp!, {r0-r12, pc}^

/*======================================================================*/
/* The Reset Entry Point				    		*/
/*======================================================================*/
.globl	Reset_Handler
Reset_Handler:                   	/* Reset Entry Point */
/************************************************************************/
/* Assembly control directives - IF, ELSE, and ENDIF			*/
/* Syntax								*/
/* IF logical-expression						*/
/* ...									*/
/* {ELSE								*/
/* ...}									*/
/* ENDIF								*/
/* Symbol [ is a synonym for IF.					*/
/* Symbol | is a synonym for ELSE					*/
/************************************************************************/
#ifdef ROM_AT_ADDRESS_ZERO
#else 
	swi 0xff 				/* Call SWI Vector  */
#endif 						/* ROM_AT_ADDRESS_ZERO */

/*=====================================*/ 
/* LED (back pannel) Display, LED D1 = P16 */ 
/*=====================================*/

INITIALIZE_LED:

/* Configure port P16 as output */

	LDR	r1, =IOPMOD			/* I/O port mode register  */
	LDR	r0, =0x10000			/* [16] I/O port mode bit for port 16,0=input,1=output */
	STR	r0, [r1]			/* write to I/0 port mode register */

/* write 0/1 to port P16 */

	LDR	r1, =IOPDATA			/* I/O port data register */
	LDR	r0, =0x00000			/* write 0 to port P16 */
	STR	r0, [r1]			/* write to I/O port data register */

/*=====================================*/
/* Setup Special Register */
/*=====================================*/

#ifdef EDO_DRAM_CONFIG  
	LDR	r0, =0x3FF0000	   		/* Read SYSCFG register value */
	LDR 	r1,[r0]		   		/* To idetify DRAM type */
	LDR	r2, =0x80000000	   
	AND	r0, r1, r2         		/* Mask DRAM type mode bit */
	CMP	r0, r2
	BNE 	EDO_DRAM_CONFIGURATION
#endif 						/* EDO_DRAM_CONFIG */
	B 	SYNC_DRAM_CONFIGURATION 	

/*==================================================*/
/* Special Register Configuration for EDO mode DRAM */
/* When KS32C5000 and KS32C50100                    */
/*==================================================*/ 
#ifdef EDO_DRAM_CONFIG
EDO_DRAM_CONFIGURATION:
	LDR	r0, =0x3FF0000		 
	LDR	r1, =0x3FFFF90   	/* SetValue = 0x3FFFF91 */
	STR	r1, [r0]	  	/* Cache,WB disable */
                           	        /* Start_addr = 0x3FF00000 */
/* ROM and RAM Configuration(Multiple Load and Store) */
        LDR     r0, =SystemInitData
	LDMIA   r0, {r1-r12}
	LDR	r0, =0x3FF0000 + 0x3010 /* ROMCntr Offset : 0x3010 */
	STMIA   r0, {r1-r12}

	LDR 	r1,=DRAM_BASE
	STR 	r1,[r1] 		/* [DRAM_BASE] = DRAM_BASE */
	LDR 	r2,[r1]			/* Read DRAM Data */
	CMP 	r2,r1
	BEQ 	INITIALIZE_C
#endif 						/* EDO_DRAM_CONFIG */
/************************************************************************/
/* Special Register Configuration for SYNC DRAM				*/
/* Only when KS32C50100							*/
/************************************************************************/

/************************************************************************/
/* System Configuration Register (SYSCFG)   				*/
/* Reset Value = 0x7FFFF91 						*/
/*									*/
/* [0]		Stall enable (SE). Must be set to zero  		*/
/* [1] 	Cache enable. 1 - cache operations are enabled  		*/
/* [2] 	Write buffer enable (WE). Then set to 1, write buffer 		*/
/* 		operatin operations are enabled			 	*/
/* [3] 	0								*/
/* [5:4]	Cacahe mode (CM)					*/
/*		00 = 4-Kbyte SRAM, 4-Kbyte cache			*/
/*		01 = 0-Kbyte SRAM, 8-Kbyte cache			*/
/*		10 = 8-kbyte SRAM, 0-Kbyte cache			*/
/*		Note: 	When you write 10 to this field, the cache 	*/
/*			enable bit is cleared automatically		*/
/* [15-6]	Internal SRAM base pointer				*/
/* [25-16	Special register bank base pointer 			*/
/* [29-26]	Product Identifier (PD_ID) 				*/
/* 		0000=KS32C5000						*/
/* 		0001=KS32C50100						*/
/* [31]	Sync DRAM Mode							*/
/* 		0 = Normal/EDO Dram interface for 4 DRAM banks		*/
/* 		1 = Sync. DRAM interface for 4 DRAM banks		*/
/************************************************************************/

/* Reset value (0x7FFFF91) 						*/
/* 									*/
/* ...0....7....F....F....F....F....9....1 				*/
/* 0000.0111.1111.1111.1111.1111.1001.0001 				*/
/* 1098.7654.3210.9876.5432.1098.7654.3210 				*/
/* .3...........2............1...........0 				*/
/* [0]     Stall enable (SE) = 1  					*/		
/* [1]     Cache enable = 0 						*/
/* [2]     Write buffer enable = 0					*/
/* [5:4]   Cache mode = 01 (no SRAM, 8-Kbyte Cache)			*/
/* [15:6]  Internal SRAM base pointer = 1111.1111.10 = 0x3FE(0x3FE0000)	*/
/* [25:16] Spesial register bank pointer=11.1111.1111= 0x3FF(0x3FF0000)	*/
/* [29:26] Product Identifier = 00.01 = KS32C50100 			*/
/* 									*/
/* Configurated value (0x83FFFFA0) 					*/
/* 									*/
/* ...8....3....F....F....F....F....A....0 				*/
/* 1000.0011.1111.1111.1111.1111.1010.0000 				*/
/* 1098.7654.3210.9876.5432.1098.7654.3210 				*/
/* .3...........2............1...........0 				*/
/* [0]     Stall enable (SE) = 0  					*/
/* [1]     Cache enable = 0  						*/
/* [2]     Write buffer enable = 0 					*/
/* [5:4]   Cache mode = 10 (10 = 8-Kbyte SRAM) 				*/
/* [15:6]  Internal SRAM base pointer = 1111.1111.10=0x3FE(0x3FE 0000)	*/
/* [25:16] Spesial register bank pointer=11.1111.1111=0x3FF(0x3FF 0000) */
/* [29:26] Product identifier (PD_ID) = KS32C5000 			*/
/* [31]    Sync DRAM interface for 4 DRAM banks 			*/

SYNC_DRAM_CONFIGURATION:
#ifdef ROM_AT_ADDRESS_ZERO
/* ROM and RAM Configuration(Multiple Load and Store) */

/* ROM sturtup tests & monitor */
   	LDR     r0, =SystemInitDataSDRAM
	LDMIA   r0, {r1-r12}
	LDR	r0, =0x3FF0000 + 0x3010 /* EXTDBWTH Offset : 0x3010 */
	STMIA   r0, {r1-r12}
#endif 						/* ROM_AT_ADDRESS_ZERO */

	LDR	r0, =0x3FF0000		 
	LDR	r1, =0x83FFFFA0   	/* SetValue = 0x83FFFFA1 */
	STR	r1, [r0]	  	/* Cache,WB disable */
                           		/* Start_addr = 0x3FF00000 */
INITIALIZE_C:
#ifdef ROM_AT_ADDRESS_ZERO
/* ROM_AT_ADDRESS_ZERO defined in ROM startup tests and in Monitor */
#ifdef LED_ONLY
/* Monitor */
/* RO area - ROM bank 0; RW, ZI in internal SDRAM */
/* => copy RW area to internal SDRAM */
    	LDR     r0, =_etext
    	LDR     r1, =_sdata
        LDR     r3, =_edata
RW_copy:
        CMP     r1, r3 			/* while(r1 < r3) */
    	LDRLO   r2, [r0], #4	     	/* [r0 += 4] */
    	STRLO   r2, [r1], #4	     	/* [r1 += 4] */
    	BLO     RW_copy
#else 
/* ROM Startup, copy  RO/RW areas to external DRAM */
        MOV     r0, #0 
        MOV     r1, #0x1000000
    	LDR     r3, =_sbss
RO_RW_copy:   	
        CMP     r0, r3          	/* while(r0 < r3) */
    	LDRLO   r2, [r0], #4	  	/* [r0 += 4] */
    	STRLO   r2, [r1], #4	 	/* [r1 += 4] */
    	BLO     RO_RW_copy
#endif 						/* LED_ONLY */ 
#endif 						/* ROM_AT_ADDRESS_ZERO */ 
	
/* Init ZI area */
    	MOV     r2, #0
    	LDR     r1, =_sbss
    	LDR     r3, =_ebss
#ifdef ROM_AT_ADDRESS_ZERO
#ifdef LED_ONLY
#else 
        ORR     r1, r1, #0x1000000
        ORR     r3, r3, #0x1000000
#endif 						/* LED_ONLY */
#endif						/* ROM_AT_ADDRESS_ZERO */
ZI_clear:  	CMP     r1, r3               
    	STRLO   r2, [r1], #4
    	BLO     ZI_clear

/************************************************************************/
/*---+------------+							*/
/*   |            |							*/
/*   |            |        +------------+ Image$$ZI$$Limit (_ebss)	*/
/*   |            |        |            |                 		*/
/*   |            |        | ZI Output  | ZI execution  		*/
/* R |            |        |    section |    region  			*/
/* A |            |        |            |                 		*/
/* M |            |        |            | Image$$ZI$$Base  (_sbss)	*/
/*   |            |        +------------+	    is equal     	*/
/*   |            |        |            | Iage$$RW$$Limit  (_edata)	*/
/*   |            |        |            |                 		*/
/*   |            |        | RW Output  | RW execution 			*/
/*   |            |        |    section |    region 			*/
/*   |            |        |            |                 		*/
/*---+------------+--------+------------+ Image$$RW$$Base  (_sdata)	*/
/*   | RW Output  |        |            |				*/
/* R |    section | single |            |  				*/
/* O +------------+ load   +------------+ Image$$RO$$Limit (_etext)    	*/
/*   |            | region |            |                 		*/
/* M | RO Output  |        | RO Output  | RO execution  		*/
/*   |    section |        |    section |    region   			*/
/*---+------------+--------+------------+ 		   (_stext)	*/	
/*    LOAD VIEW            EXECUTION VIEW				*/
/************************************************************************/

#ifdef ROM_AT_ADDRESS_ZERO
#ifdef LED_ONLY
/* Monitor, DRAM remain start from 0x01000000 (16M-32M)*/
#else 
/* ROM startup test, remap DRAM to 0x00000000, ROM to 0x02000000 */
   	LDR     r0, =SystemWorkDataSDRAM
	LDMIA   r0, {r1-r12}
	LDR	r0, =0x3FF0000 + 0x3010 	/* EXTDBWTH Offset : 0x3010 */
	STMIA   r0, {r1-r12}
#endif 						/* LED_ONLY */
#endif						/* ROM_AT_ADDRESS_ZERO */


/* Call C_Entry application routine with a pointer to the first */
/* available memory address after ther compiler's global data   */
/* This memory may be used by the application.                  */
	/*===========================*/
	/* Now we enter the C Program*/
	/*===========================*/

/* UART0 Baud Rate Register */   
	LDR	r1, =UARTBRD0			/* I/O port data register */
        LDR     r2, =0x00280       		/* 38400 */
/* 0x00a20 //   9600 // cnt0 = 162 cnt1 = 0, 0 */
/* 0x00500 //  19200 // cnt0 =  80 cnt1 = 0, 1 */
/* 0x00280 //  38400 // cnt0 =  40 cnt1 = 0, 2 */
/* 0x001a0 //  57600 // cnt0 =  26 cnt1 = 0, 3 */
/* 0x000d0 // 115200 // cnt0 =  13 cnt1 = 0, 4 */
/* 0x00060 // 230400 // cnt0 =   6 cnt1 = 0, 5 */
/* 0x00020 // 460800 // cnt0 =   2 cnt1 = 0, 6 not available*/
        STR     r2, [r1]

	LDR	r0, =0x3FF0000	   		/* SYSCFG register address */
	LDR 	r1,[r0]		   		/* Read SYSCFG register value */
	LDR	r2, =0x0C000000	   		/* lo bits of PID(product id) */
/* [30:26] */
/* 00000 = KS32C5000 */
/* 00001 = KS32C50100 s3c4510x */
/* 11001 = s3c4510B */
/* 00011 = s3c4530X */
/* 10011 = s3c4530A */
	AND	r0, r1, r2         		
	CMP	r0, r2
	BEQ 	UART0_S3C4530_CONF

UART0_S3C4510_CONF:
/* UART0 Line Control Register */
	LDR	r1, =UARTLCON0			/* I/O port data register */
        LDR     r2, =0x03        		/* ULCON_WL8 0x03, word lengt 8 bit */
        STR     r2, [r1]
/* UART0 Control register */    
	LDR	r1, =UARTCONT0			/* I/O port data register */
        LDR     r2, =0x09        		/* Tx mode int, Rx mode int */
        STR     r2, [r1]
	B 	P16_start

UART0_S3C4530_CONF:
	LDR	r1, =UCON0			/* I/O port data register */
        LDR     r2, =0x00003005       		/* word len 8 bit, tx mode int, rx mode int */
        STR     r2, [r1]

P16_start:

/* write 0/1 to port P16 */

////////////////////////////////////////////////////////////////////////////////
        LDR     r3, =0x4	

P16_write:
	LDR	r0, =0x00000			/* write 0 to port P16 */
	LDR	r1, =IOPDATA			/* I/O port data register */
	MOV	r2, #0x040000			/* loop counter */

P16_write_0:
	STR	r0, [r1]			/* 1 cycle, write to I/O port data register */
	SUBS	r2, r2, #1			/* 1 cycle, Down Count */
	BNE 	P16_write_0			/* 3 cycles */

	LDR	r0, =0x10000			/* write 1 to port P16 */
	MOV	r2, #0x040000			/* loop counter */

P16_write_1:

	STR	r0, [r1]			/* write to I/O port data register */
	SUBS	r2, r2, #1			/* Down Count */
	BNE 	P16_write_1	

        SUBS    r3, r3, #1
        BNE     P16_write

/************************************************************************/
/* Initialise STACK 							*/
/************************************************************************/

INITIALIZE_STACK:
	MRS	r0, cpsr			/* r0 = cpsr */
	BIC	r0, r0, #LOCK_MASK | MODE_MASK	/* clear I,F, mode bits */
	ORR	r2, r0, #USR_MODE		/* r2 : I = 0, F = 0, M[4:0] = USER_MODE  */

/* operand2 = <#expression>, assembler will attept to generate a */
/* shifted immediate 8-bit field to match the expression. If this */
/* is imposible, it will give an error. */

	LDR 	r4, =_end 

/* Undefined instruction cpsr, spsr, sp */
	ORR	r1, r0, #LOCK_MASK | UDF_MODE	/* I=1,F=1,M=UDF_MODE */
	MSR	cpsr_csxf, r1			/* cpsr = r1 */
	MSR	spsr_csxf, r2			/* spsr (saved CPSR = USER mode, enable FIQ, IRQ) */
	add     r4, r4, #UDF_STACK_SIZE
#ifndef LED_ONLY
	str 	r4, UDF_STACK
#endif 						/* LED_ONLY */
	mov	sp, r4				/* undefined mode stack */

/* Abort mode cpsr, spsr, sp*/
	ORR	r1, r0, #LOCK_MASK | ABT_MODE	/* I=1,F=1,M=ABT=MODE */
	MSR	cpsr_csxf, r1			/* cpsr (enter ABORT mode, disable FIQ, IRQ) */
	MSR	spsr_csxf, r2			/* spsr (saved CPSR = USER mode, enable FIQ, IRQ) */
	add     r4, r4, #ABT_STACK_SIZE
#ifndef LED_ONLY
	str 	r4, ABT_STACK
#endif 								/* LED_ONLY */
	mov	sp, r4				/* Abort Mode stack (defined in STACK area) */


/* Supervisor mode cpsr, spsr, sp */
	ORR	r1, r0, #LOCK_MASK | SUP_MODE	/* I=1, F=1,M=SUP_MODE */
	MSR	cpsr_csxf, r1			/* cpsr */
	MSR	spsr_csxf, r2			/* spsr */
	add     r4, r4, #SUP_STACK_SIZE
#ifndef LED_ONLY
	str 	r4, SUP_STACK
#endif 								/* LED_ONLY */
	mov	sp, r4   			/* Change CPSR to SVC mode */

/* IRQ cpsr, spsr, sp */
	ORR	r1, r0, #LOCK_MASK | IRQ_MODE	/* I=1,F=1,M=IRQ_MODE */
	MSR	cpsr_csxf, r1			/* cpsr (enter IRQ mode, disable FIQ, IRQ) */
	MSR	spsr_csxf, r2			/* spsr (saved CPSR = USER mode, enable FIQ,IRQ) */ 
	add     r4, r4, #IRQ_STACK_SIZE
#ifndef LED_ONLY
	str 	r4, IRQ_STACK
#endif 								/* LED_ONLY */
	mov	sp, r4				/* IRQ stack (defined below in STACK area) */

/* FIQ cpsr, spsr, sp */
	ORR	r1, r0, #LOCK_MASK | FIQ_MODE	/* I=1,F=1,M=FIQ_MODE */
	MSR	cpsr_csxf, r1			/* cpsr = r1 (change mode=FIQ, disable FIQ,IRQ) */
	MSR	spsr_csxf, r2			/* spsr = r2 (I=0,F=0,M=USER_MODE)   */
	add     r4, r4, #FIQ_STACK_SIZE
#ifndef LED_ONLY
	str 	r4, FIQ_STACK
#endif 								/* LED_ONLY */
	mov	sp, r4				/* FIQ stack (defined below in STACK area) */

/* setup USR stack */
	MSR	cpsr_csxf, r2
	add     r4, r4, #USR_STACK_SIZE
#ifndef LED_ONLY
	str 	r4, USR_STACK
#endif 								/* LED_ONLY */
	mov	sp, r4

/* now in user mode */
        BL      C_Entry		

#ifdef ROM_AT_ADDRESS_ZERO
/*======================================================*/
/* DRAM System Initialize Data(KS32C5000 and KS32C50100)*/ 
/*======================================================*/
SystemInitData:
#ifdef EDO_DRAM_CONFIG 
	.word rEXTDBWTH		/* */
	.word initROMCON0	/* */
	.word initROMCON1	/* */ 
    	.word initROMCON2	/* */
	.word initROMCON3	/* */
	.word initROMCON4	/* */
	.word initROMCON5	/* */ 
	.word initDRAMCON0   	/* */
	.word initDRAMCON1	/* */
	.word initDRAMCON2	/* */
	.word initDRAMCON3	/* */
	.word rREFEXTCON  	/* */
#endif 						/* EDO_DRAM_CONFIG */
/*====================================================*/
/*DRAM System Initialize Data	(KS32C50100 only)     */
/*====================================================*/
SystemInitDataSDRAM:
	.word rEXTDBWTH		/* */
	.word initROMCON0	/* */
	.word initROMCON1	/* */
    	.word initROMCON2	/* */
	.word initROMCON3	/* */
	.word initROMCON4	/* */
	.word initROMCON5	/* */
	.word initSDRAMCON0  	/* */
	.word initSDRAMCON1	/* */
	.word initSDRAMCON2	/* */
	.word initSDRAMCON3	/* */
	.word rSREFEXTCON 	/* */

#ifdef LED_ONLY
/* Monitor used only SystemInitDataSDRAM memory map */
#else 
SystemWorkDataSDRAM:
	.word rEXTDBWTH		/* */
	.word workROMCON0	/* */
	.word workROMCON1	/* */
    	.word workROMCON2	/* */
	.word workROMCON3	/* */
	.word workROMCON4	/* */
	.word workROMCON5	/* */
	.word workSDRAMCON0  	/* */
	.word workSDRAMCON1	/* */
	.word workSDRAMCON2	/* */
	.word workSDRAMCON3	/* */
	.word rSREFEXTCON 	/* */
#endif						/* LED_ONLY */
#endif 						/* ROM_AT_ADDRESS_ZERO */ 

#ifdef LED_ONLY
#else 
.globl StartDownPgm
StartDownPgm:
	MOV	PC, r0

.globl UDF_STACK
UDF_STACK: 
    .word 0x0badc0de

.globl ABT_STACK 
ABT_STACK:
    .word 0x0badc0de

.globl SUP_STACK
SUP_STACK: 
    .word 0x0badc0de

.globl IRQ_STACK
IRQ_STACK: 
    .word 0x0badc0de

.globl FIQ_STACK
FIQ_STACK: 
    .word 0x0badc0de

.globl USR_STACK
USR_STACK: 
    .word 0x0badc0de
#endif						/* LED_ONLY */


