/*************************************************************************/
/*                                                                       */
/* FILE NAME                                      VERSION                */
/*                                                                       */
/*      uart.c                     KS32C5000(A)/50100  SNDS100 Ver1.0    */
/*                                                                       */
/* COMPONENT                                                             */
/*                                                                       */
/*                                                                       */
/* DESCRIPTION                                                           */
/*                                                                       */
/*      UART library functions.                                          */
/*                                                                       */
/* AUTHOR                                                                */
/*                                                                       */
/*      Young Sun KIM, Samsung Electronics, Inc.                         */
/*                                                                       */
/* DATA STRUCTURES                                                       */
/*                                                                       */
/*                                                                       */
/* FUNCTIONS                                                             */
/*                                                                       */
/*       UART_Initialize                    Initialize serial driver     */
/*       i_getc                             Get a character from buffer  */
/*       i_gets                             Get a string from buffer     */
/*       i_putc                             print a character            */
/*       i_puts                             Print a string               */
/*       i_printf                           formmatted print             */
/*                                                                       */
/* DEPENDENCIES                                                          */
/*                                                                       */
/*                                                                       */
/* HISTORY                                                               */
/*                                                                       */
/*         NAME            DATE                    REMARKS               */
/*                                                                       */
/*         in4maker     05-31-1999      Created initial version 1.0      */
/*                                                                       */
/*************************************************************************/

/* 			*/
/* Modified by 		*/
/* Dmitriy Cherkashin 	*/
/* dch@ucrouter.ru	*/
/* 2002,2003		*/
/*			*/

#include "evm50100.h"		// evm50100 board definition 	
#include "ks32c50.h"

#define CHECK_UART

////////////////////////////////////////////////////////////////////////////////
// INITIALIZE UART GLOBAL VARIABLE /////////////////////////////////////////////
////////////////////////////////////////////////////////////////////////////////

static U8 CONSOLE_last = 0;	// last received from console character

// Transmit & Receive Que data structure ///////////////////////////////////////
#define MAXEVENT 8192  		// buffer size 

typedef struct 
{
 U8 buff[MAXEVENT];    		// data buffer
 volatile int wptr; 		// write pointer
 volatile int rptr;    		// read pointer
 volatile int cnt;		// bytes count
}
 UART_BUFFER;

/* Define references to external structures */
UART_BUFFER  RxQ[2];		// Receive Queue
UART_BUFFER  TxQ[2]; 		// Transmit Queue

struct 
{
 U32 overrun; 			// overrun error
 U32 parity;			// parity error
 U32 frame;			// frame error
 U32 brdet;			// break
}
 UART_ERR[2];


#define BAUD_TABLE_SIZE  7      // No of baud rate table

struct 
{
 U32 baud;			// baud rate
 U32 div;			// divisor 
}
 BaudTable[BAUD_TABLE_SIZE] = 
{
#ifdef EX_UCLK
/* for 29.4912MHz UART clock */
   9600, 0x000bf,
  19200, 0x0005f,
  38400, 0x0002f,
  57600, 0x0001f,
 115200, 0x0000f,
 230400, 0x00007,
 460800, 0x00003
#else
#ifdef _KS32C50100_
/* for 50MHz/2 UART clock */
/* see KS32C50100 RISC MICROCONTROLLER reference, page 10-14 */
   9600, 0x00a20,		// cnt0 = 162 cnt1 = 0, 0
  19200, 0x00500,		// cnt0 =  80 cnt1 = 0, 1
  38400, 0x00280,		// cnt0 =  40 cnt1 = 0, 2
  57600, 0x001a0,		// cnt0 =  26 cnt1 = 0, 3
 115200, 0x000d0,		// cnt0 =  13 cnt1 = 0, 4
 230400, 0x00060,		// cnt0 =   6 cnt1 = 0, 5
 460800, 0x00020   		// cnt0 =   2 cnt1 = 0, 6 not available 
/* UBRDIV0, UBRDIV1 format 					*/
/* [ 0:3] = Baud rate divisor value CNT1 			*/
/* 	    xxx0 = divide by 1					*/	
/* [15:4] = Time constant value for CNT0 			*/
/* BRGOUT = (MCLK2 or UCLK) / (CNT0 + 1) / (16^CNT1) / 16 	*/
#else
   9600, 0x00a20,
  19200, 0x00500,
  38400, 0x00280,
  57600, 0x001a0,
 115200, 0x000d0,
 230400, 0x00060,
 460800, 0x00020 
#endif
#endif
};

////////////////////////////////////////////////////////////////////////////////
// UART0 Transmit Interrupt Service Routine ////////////////////////////////////
////////////////////////////////////////////////////////////////////////////////

#ifdef _KS32C50100_
////////////////////////////////////////////////////////////////////////////////
// KS32C50100 UART channel 0 Tx int service routine ////////////////////////////
////////////////////////////////////////////////////////////////////////////////
void S3C4510_Uart0TxISR(void)
{
 U32 stat = UARTSTAT0;			// read UART0 status register
 if((stat & UARTSTAT_TXB_EMPTY)!= 0) 	/* Uart 0 Tx Buffer Empty */
  {
   if(TxQ[0].cnt > 0) 
    {					// tx queue not empty
     if(TxQ[0].rptr >= MAXEVENT) { 	// set read pointer to queue start 
       TxQ[0].rptr = 0; 		
     }
     UARTTXH0=TxQ[0].buff[TxQ[0].rptr++];// write to UART0 transmit holding regs
     TxQ[0].cnt--;			// remain bytes in UART0 transmit queue
    }
  }
//
 if((stat & UARTSTAT_ERROR) != 0) 
  {
// overrun, parity, frame bits automaticaly cleared when status register is readed
   if((stat & UARTSTAT_OVERRUN)) { 	// overrun error
     UART_ERR[0].overrun++; 		// calculate overrun error count
   }
   if((stat & UARTSTAT_PARITY))	 {	// parity error 
     UART_ERR[0].parity++;		// calculate parity error count
   }
   if((stat & UARTSTAT_FRAME))	 { 	// frame error
     UART_ERR[0].frame++;		// calculate frame error count
   } 
  }
 if((stat & UARTSTAT_BREAK))	 {	// calculate break detect count
   UART_ERR[0].brdet++;			// 
  } 
}

////////////////////////////////////////////////////////////////////////////////
// KS32C50100 UART1 Tx ISR /////////////////////////////////////////////////////
////////////////////////////////////////////////////////////////////////////////

void S3C4510_Uart1TxISR(void)
{
 U32  stat = UARTSTAT1;			// read status register 0
 if( (stat & UARTSTAT_TXB_EMPTY) != 0 ) 
  {					// Uart 0 Tx Buffer Empty
   if(TxQ[1].cnt > 0) {
     if(TxQ[1].rptr >= MAXEVENT) { 	// read pointer out off queue buffer
       TxQ[1].rptr = 0; 		// set read pointer to bugger start
     }
     UARTTXH1=TxQ[1].buff[TxQ[1].rptr++];// write byte to UART1 tx holding register
     TxQ[1].cnt--;			// remain bytes in tx queue
   }
  }
//
 if((stat & UARTSTAT_ERROR) != 0) 
  {
// overrun, parity, frame bits automaticaly cleared when status register is readed
   if((stat & UARTSTAT_OVERRUN)) { 	// overrun error
     UART_ERR[1].overrun++; 		// calculate overrun error count
   }
   if((stat & UARTSTAT_PARITY))	 {	// parity error 
     UART_ERR[1].parity++;		// calculate parity error count
   }
   if((stat & UARTSTAT_FRAME))	 { 	// frame error
     UART_ERR[1].frame++;		// calculate frame error count
   } 
  }
 if((stat & UARTSTAT_BREAK))	 {
   UART_ERR[1].brdet++;		
  } 
}
#endif								/* _KS32C50100_ */


#ifdef _S3C4530_
////////////////////////////////////////////////////////////////////////////////
// S3C4530 UART channel 0 Tx int service routine ///////////////////////////////
////////////////////////////////////////////////////////////////////////////////
void S3C4530_Uart0TxISR(void)
{
 if((USTAT0 & USTAT_THE) != 0) 
  { 					// Uart Tx holding register Empty
   if(TxQ[0].cnt > 0) 
    {					// UART0 Tx queue not empty
     if(TxQ[0].rptr >= MAXEVENT) { 	// set read pointer to 
       TxQ[0].rptr = 0; 		// start off queue buffer
     }
     UARTTXH0=TxQ[0].buff[TxQ[0].rptr++];// write to uart0 tx holding register 
     TxQ[0].cnt--;			// remain bytes in UART0 tx queue 
    }
  }
}

////////////////////////////////////////////////////////////////////////////////
// S3C4530 UART channel 1 Tx int service routine ///////////////////////////////
////////////////////////////////////////////////////////////////////////////////
void S3C4530_Uart1TxISR(void)
{
 if((USTAT1 & USTAT_THE) != 0) 
  { 					// Uart1 Tx Holding Register Empty
   if(TxQ[1].cnt > 0) 
    {					// UART1 Tx queue not empty
     if(TxQ[1].rptr >= MAXEVENT) { 	// set read pointer 
       TxQ[1].rptr = 0; 		// to begin 
     }
     UARTTXH1=TxQ[1].buff[TxQ[1].rptr++];// write to UART1 tx holding register
     TxQ[1].cnt--;			// remain bytes in tx queue
    }
  }
}
#endif


////////////////////////////////////////////////////////////////////////////////
// UART0 Rcv, Error Interrupt Service Routine //////////////////////////////////
////////////////////////////////////////////////////////////////////////////////

#ifdef _KS32C50100_
////////////////////////////////////////////////////////////////////////////////
// S3C4510 (KS32C50100) Receive, Error Interrupt Service Routine ///////////////
////////////////////////////////////////////////////////////////////////////////
void S3C4510_Uart0RxErrISR(void)
{
 U32 stat;				// status register content 	
 stat = UARTSTAT0;			// read UART0 status register
 if((stat & UARTSTAT_RCV_READY) != 0) 
  {					// UARTRXB0 contain valid rx data
   U8 ch = UARTRXB0;			// read received byte 
   if(RxQ[0].cnt < MAXEVENT)
    {					// UART0 queue not fill
     if(RxQ[0].wptr >= MAXEVENT) { 	// set write pointer 
       RxQ[0].wptr = 0; 		// to begin of queue 	
     }
// read UART0 rx buffer register
     RxQ[0].buff[RxQ[0].wptr++] = ch;	
     RxQ[0].cnt++;			// bytes in rx queue
    } 
   else
    {					// UART 0 Rx queue full
     UART_ERR[0].overrun++; 		// calculate overrun error count
    }  
  }

 if((stat & UARTSTAT_ERROR) != 0) 
  {
// overrun, parity, frame bits automaticaly cleared when status register is readed
   if((stat & UARTSTAT_OVERRUN)) {	// overrun error
     UART_ERR[0].overrun++; 		// calculate overrun error count
   }
   if((stat & UARTSTAT_PARITY))	 {	// parity error 
     UART_ERR[0].parity++;		// calculate parity error count
   }
   if((stat & UARTSTAT_FRAME))	 { 	// frame error
     UART_ERR[0].frame++;		// calculate frame error count
   } 
  }
 if((stat & UARTSTAT_BREAK))	 { 					
   UART_ERR[0].brdet++;		
  } 
}

////////////////////////////////////////////////////////////////////////////////
// KS32C50100 rX Int ///////////////////////////////////////////////////////////
////////////////////////////////////////////////////////////////////////////////

void S3C4510_Uart1RxErrISR(void)
{
 U32 stat;				// status register content 	
 stat = UARTSTAT1;			// read UART0 status register
 if((stat & UARTSTAT_RCV_READY) != 0) 
  {					// UARTRXB1 contain valid rx data  
   U8 ch = UARTRXB1; 			
   if(RxQ[1].cnt < MAXEVENT)
    {					// UART1 Rx Queue no full
     if(RxQ[1].wptr >= MAXEVENT) { 	// set write pointer 
       RxQ[1].wptr = 0; 		// to begin of queue
     }
// read from channel 0 
     RxQ[1].buff[RxQ[1].wptr++] = ch;	
     RxQ[1].cnt++;			// bytes in UART1 rx queue
    } 
   else
    {					// UART1 rx queue full 
     UART_ERR[1].overrun++; 		// calculate overrun error count
    } 
  }

 if((stat & UARTSTAT_ERROR) != 0) 
  {
// overrun, parity, frame bits automaticaly cleared when status register is readed
   if((stat & UARTSTAT_OVERRUN)) {	// overrun error
     UART_ERR[1].overrun++; 		// calculate overrun error count
   }
   if((stat & UARTSTAT_PARITY))	 {	// parity error 
     UART_ERR[1].parity++;		// calculate parity error count
   }
   if((stat & UARTSTAT_FRAME))	 { 	// frame error
     UART_ERR[1].frame++;		// calculate frame error count
   } 
  }

 if((stat & UARTSTAT_BREAK))	 { 					
   UART_ERR[1].brdet++;		
 } 
}
#endif 							/* _KS32C50100_ */


#ifdef _S3C4530_
////////////////////////////////////////////////////////////////////////////////
// S3C4530 Receive Error Interrupt Service Routine ///////////////////////////// 
////////////////////////////////////////////////////////////////////////////////
void S3C4530_Uart0RxErrISR(void)
{
 U32 stat, wstat = 0;			// readed status, write  
 stat = USTAT0;				// read UART0 status 
 if((stat & USTAT_OER) != 0) {		// [4] Overrun Error (OER)
   wstat |= USTAT_OER;			// set write bit 
   UART_ERR[0].overrun++;		// calc UART0 overrun errors count
 }
 if((stat & USTAT_BSD) != 0) { 		// [1] Break Signal Detected (BSD)
   wstat |= USTAT_BSD;			//     set write bit
   UART_ERR[0].brdet++;			// calc statistics
 } 
 if((stat & USTAT_FER) != 0) { 		// [2] Frame Error (FER)
   wstat |= USTAT_FER;			//     set write bit
   UART_ERR[0].frame++;			
 } 
 if((stat & USTAT_PER) != 0) {		// [3] Parity Error (PER)
   wstat |= USTAT_PER;
   UART_ERR[0].parity++;
 }

/* write to status register to clear bits */
 USTAT0 = wstat;
 if( (stat & USTAT_RDV) != 0 ) 
  {					// UARTRXB0 contain valid rx data
   U8 ch = UARTRXB0;			// read rx byte 
   if(RxQ[0].cnt < MAXEVENT) 
    {					// UART0 rx queue not full 
     if(RxQ[0].wptr >= MAXEVENT) { 	// set write pointer 
       RxQ[0].wptr = 0; 		// to queue begin
     } 
     RxQ[0].buff[RxQ[0].wptr++] = ch;   // read ex data & write to rx queue 
     RxQ[0].cnt++;			// bytes in UART0 rx queue
    } 
   else 
    {					// UART0 rx queue full 
     UART_ERR[0].overrun++;		// calc errors count
    }
  }
} 

void S3C4530_Uart1RxErrISR(void)
{
 U32 stat, wstat = 0;			// readed status, writed  
 stat = USTAT1;				// read UART1 status register
 if((stat & USTAT_OER) != 0) {		// [4] Overrun Error (OER)
   wstat |= USTAT_OER;			//     set write bit
   UART_ERR[1].overrun++;		// calc UART1 overrun count 
 }
 if((stat & USTAT_BSD) != 0) { 		// [1] Break Signal Detected (BSD)
   wstat |= USTAT_BSD;			// set write bit
   UART_ERR[1].brdet++;			// calc UART1 stat	
 } 
 if((stat & USTAT_FER) != 0) { 		// [2] Frame Error (FER)
   wstat |= USTAT_FER;			// set write bit
   UART_ERR[1].frame++;	
 } 
 if((stat & USTAT_PER) != 0) {		// [3] Parity Error (PER)
   wstat |= USTAT_PER;			// set write bit	
   UART_ERR[1].parity++;
 }

/* write to status register to clear status register bits */
 USTAT1 = wstat;

 if( (stat & USTAT_RDV) != 0 ) 
  {					// UARTRXB1 contain valid rx data 
   U8 ch = UARTRXB1;			// read and discard rx byte
   if(RxQ[1].cnt < MAXEVENT) 
    {					// UART1 Rx queue not full
     if(RxQ[1].wptr >= MAXEVENT) { 	// set write pointer to begin of queue
       RxQ[1].wptr = 0; 		
     } 
     RxQ[1].buff[RxQ[1].wptr++] = ch;   // read UART1 rx data & write to rx queue
     RxQ[1].cnt++;			// UART1 rx queue bytes count 
    } 
   else 
    {					// UART1 Rx queue full  
     UART_ERR[1].overrun++;		// calc overrun errors count 
    }
  }
} 
#endif							// _S3C4530_

////////////////////////////////////////////////////////////////////////////////
// Initialization of global structures /////////////////////////////////////////
////////////////////////////////////////////////////////////////////////////////


U32  UART_Initialize(void)
{
 U32 uartbrd;				// UARTBRD content 
 int pos;				// position in UART Tx & Rx buffer
 int cn;				// UART channel 

//
 Disable_Int(nGLOBAL_INT);  		// Global interrupt disabled 

// Disable UART0 & UART1 interrupts ////////////////////////////////////////////
 UARTRxIntOff(0);			// UART0 Rx interrupt disable
 UARTRxIntOff(1);			// UART1 Rx interrupt disable
 UARTTxIntOff(0);			// UART0 Tx interrupt disable 
 UARTTxIntOff(1);			// UART1 Tx interrupt disable

// Initialize UART0&UART1 transmit & receive Queue /////////////////////////////

 for(cn = 0; cn < 2; ++cn)		// 
  {
   TxQ[cn].wptr = 0;			// Tx Queue write pointer 	
   TxQ[cn].rptr = 0;			// Tx Queue read pointer 
   TxQ[cn].cnt  = 0;			// Tx Queue bytes count

//
   RxQ[cn].wptr = 0;			// Rx Queue write pointer 
   RxQ[cn].rptr = 0;			// Rx Queue read pointer 
   RxQ[cn].cnt  = 0;			// Rx Queue bytes count

   UART_ERR[cn].overrun = 0; 		// clear overrun error count
   UART_ERR[cn].parity  = 0;		// clear parity error count
   UART_ERR[cn].frame   = 0;		// clear frame error count
   UART_ERR[cn].brdet   = 0;		// break detected


// clear UART0/UART1 Rx & Tx buffers ///////////////////////////////////////////  
   for(pos = 0; pos < MAXEVENT; ++pos) 
    {
     TxQ[cn].buff[pos] = 0xFF;
     RxQ[cn].buff[pos] = 0xFF;
    }
  }

////////////////////////////////////////////////////////////////////////////////
 for(uartbrd = 0; uartbrd < BAUD_TABLE_SIZE; ++uartbrd) 
  {
    if(BaudTable[uartbrd].baud == baudrate) {			
      break;				// 
    }
  }

 if(uartbrd < BAUD_TABLE_SIZE)
  {
// baudrate find in table //////////////////////////////////////////////////////
   uartbrd = BaudTable[uartbrd].div;
  }
 else
  {
   uartbrd = BaudTable[0].div;
  }

// 
 UARTBRD0  = uartbrd; 			// UART0 baud rate register
 UARTBRD1  = uartbrd; 			// UART1 baud rate register 

#ifdef _KS32C50100_
// system compiled with KS32C50100 support /////////////////////////////////////
 if((SYSCFG & 0x0C000000) != 0x0C000000) 
  {
   U32 stat;
// (product id != S3C4530) => KS32C50100 ///////////////////////////////////////
// UARTLCON0 & UARTLCON1 content // (Uart Line control register content) /////// 
   UARTLCON1 = UARTLCON0 = 		// 
    UARTLCON_WL8    | 			// word length = 8 bits
    UARTLCON_PMD_NO | 			// parity mode = no oarity 
#ifdef EX_UCLK  
    UARTLCON_UCLK;      		// external clock,29.4912MHz
#else
    0;           			// internal clock
#endif
// UARTCONT0 & UARTCONT1 content /////////////////////////////////////////////// 
   UARTCONT1 = UARTCONT0 = 
    UARTCON_RXM_INTREQ | 		// rx mode = int req
    UARTCON_TXM_INTREQ |		// tx mode = int req	
    UARTCON_RXSTAT_INT;			// Rx Status () Interrupt

   SysSetInterrupt(nUART0_TX_INT    , S3C4510_Uart0TxISR);    // UART 0 tx int 
   SysSetInterrupt(nUART1_TX_INT    , S3C4510_Uart1TxISR);    // UART 1 tx int
   SysSetInterrupt(nUART0_RX_ERR_INT, S3C4510_Uart0RxErrISR); // UART 0 rx int 
   SysSetInterrupt(nUART1_RX_ERR_INT, S3C4510_Uart1RxErrISR); // UART 1 rx int  

   stat = UARTSTAT0;
   stat = UARTSTAT1;
  }
#endif

#ifdef _S3C4530_
// compiled with S3C4530 support ///////////////////////////////////////////////
 if((SYSCFG & 0x0C000000) == 0x0C000000) 
  {
   U32 stat;
// lo bits of PID = 11 => s3C4530 ////////////////////////////////////////////// 
   UCON1 = UCON0 = 
    UCON_TMODE_INTREQ | 		// transmit mode = int request 
    UCON_RMODE_INTREQ | 		// receive mode = int request 
#ifdef EX_UCLK  
    UCON_UCLK         |
#endif
    UCON_TFEN  	      |			// transmit fifo enable 
    UCON_RFEN  	      |			// receive  fifo enable
    UCON_TFRST 	      |			// transmit fifo reset 
    UCON_RFRST        |			// receive  fifo reset 
    UCON_RFTL_1	      | 		/* 00 = 1-byte vaild/32-byte */
    UCON_TFTL_8       |			/* 00 = 30-byte empty/32-byte */		
    UCON_WL8;				// word length = 8 bit


//
   UINTEN1 = UINTEN0 = 
    UINTEN_THEIE |			/* enable transmit holding reg empty */ 
    UINTEN_RDVIE |			/* Enable UART1 Rx Interrupt */
    UINTEN_BSDIE |			/* break signal detected */
    UINTEN_FERIE |			// frame error int en
    UINTEN_PERIE |			// parity error int en
    UINTEN_OERIE; 			// overrun int en

   SysSetInterrupt(nUART0_TX_INT    , S3C4530_Uart0TxISR);    // UART 0 tx int 
   SysSetInterrupt(nUART1_TX_INT    , S3C4530_Uart1TxISR);    // UART 1 tx int
   SysSetInterrupt(nUART0_RX_ERR_INT, S3C4530_Uart0RxErrISR); // UART 0 rx int 
   SysSetInterrupt(nUART1_RX_ERR_INT, S3C4530_Uart1RxErrISR); // UART 1 rx int  

   stat = USTAT0;
   stat = USTAT1;
  } 
#endif

//
 Enable_Int(nGLOBAL_INT);  		// Global interrupt disabled
 return(1);
}


////////////////////////////////////////////////////////////////////////////////
// set UART0/1 speed ///////////////////////////////////////////////////////////
////////////////////////////////////////////////////////////////////////////////


int UARTSetSpeed(U32 channel, int speed)
{
 U32 pos;				
 if(channel > 1)
  {
   return(0);
  } 

 for(pos = 0; pos < BAUD_TABLE_SIZE; ++pos) 
  {
   if(BaudTable[pos].baud == speed)
    {			
// baudrate find in table //////////////////////////////////////////////////////
     pos = BaudTable[pos].div;

     if(channel == 0)
      {
       UARTBRD0 = pos; 			// UART0 baud rate register
      }
     else
      {
       UARTBRD1 = pos; 			// UART1 baud rate register 
      }

//
     return(1);
    }
  }

 return(0);
}


int UARTGetSpeed(U32 channel)
{
 U32 uartbrd;				// UARTBRD content 
 U32 pos;				// BRD table index 

//
 if(channel == 0)
  {
   uartbrd = UARTBRD0; 			// UART0 baud rate register
  }
 else
 if(channel == 1)
  {
   uartbrd = UARTBRD1; 			// UART1 baud rate register 
  }
 else
  {
   return(0);
  } 


 for(pos = 0; pos < BAUD_TABLE_SIZE; ++pos) 
  {
   if(BaudTable[pos].div == uartbrd)
    {			
     return(BaudTable[pos].baud);
    }
  }

//
 return(0);
}

////////////////////////////////////////////////////////////////////////////////
// wait transmit complete //////////////////////////////////////////////////////
////////////////////////////////////////////////////////////////////////////////

int UARTWaitTxComp(int ch)
{
 if(ch == 0) 
  {
#ifdef _S3C4530_
    if(is_s3c4530(syscfg_pd_id)) {
      while( (USTAT0 & USTAT_THE) == 0 ) {;}
    } else {
      while( (UARTSTAT0 & UARTSTAT_TXB_EMPTY) == 0 ) {;}
    }
#else 
      while( (UARTSTAT0 & UARTSTAT_TXB_EMPTY) == 0 ) {;}
#endif
  }
 else 
 if(ch == 1)
  {
#ifdef _S3C4530_
    if(is_s3c4530(syscfg_pd_id)) {
      while( (USTAT1 & USTAT_THE) == 0 ) {;}
    } else {
      while( (UARTSTAT1 & UARTSTAT_TXB_EMPTY) == 0 ) {;}
    }
#else 
      while( (UARTSTAT1 & UARTSTAT_TXB_EMPTY) == 0 ) {;}
#endif
  } 
 else
  {
   return(0);	
  }
// 
 return(1);
}

//////////////////////////////////////////////////////////////////////////////////
// set UART channel loop back mode ///////////////////////////////////////////////
////////////////////////////////////////////////////////////////////////////////
// ch = channel number
int UARTSetLoopBack(int ch, int on)
{
 if(ch == 0) 
  {
// UART0 ///////////////////////////////////////////////////////////////////////
   if(on == 1) 
    {					// UART0 set loop back mode 
       U32 err;
#ifdef _S3C4530_ 	
     if(is_s3c4530(syscfg_pd_id)) {	// S3C4530 family
       UCON0 |= UCON_LOOPB | UCON_TFRST | UCON_RFRST;
     } else {
       UARTCONT0 |= UARTCON_LOOPBACK;	// KS32C50100
     }
#else
       UARTCONT0 |= UARTCON_LOOPBACK;
#endif
       for(err = 0; err < 10000; ++err) {;}
       err = UARTRXB0;
    } 
   else 
    {					// UART0 clear loop back mode 
#ifdef _S3C4530_ 	
     if(is_s3c4530(syscfg_pd_id)) {	// S3C4530 family
       UCON0     &=~UCON_LOOPB;
     } else {
       UARTCONT0 &=~UARTCON_LOOPBACK;
     }
#else
       UARTCONT0 &=~UARTCON_LOOPBACK;
#endif
    }
// end UART0 processing ////////////////////////////////////////////////////////
  }
 else
 if(ch == 1)
  {					
// UART1 ///////////////////////////////////////////////////////////////////////
   if(on == 1) 
    {					// UART1 set loop back mode 
       U32 err;
#ifdef _S3C4530_
     if(is_s3c4530(syscfg_pd_id)) {	// S3C4530 family
       UCON1 |= UCON_LOOPB | UCON_TFRST | UCON_RFRST;
     } else {
       UARTCONT1 |= UARTCON_LOOPBACK;	// 
     }
#else
     UARTCONT1 |= UARTCON_LOOPBACK;	//
#endif								/* _S3C4530_ */
       for(err = 0; err < 10000; ++err) {;}
       err = UARTRXB1;
    } 
   else 
    {					// UART1 clear loop back mode 
#ifdef _S3C4530_
     if(is_s3c4530(syscfg_pd_id)) {	// S3C4530 family
       UCON1     &=~UCON_LOOPB;
     } else {
       UARTCONT1 &=~UARTCON_LOOPBACK;
     }
#else
       UARTCONT1 &=~UARTCON_LOOPBACK;
#endif								/* _S3C4530_ */ 
    }
// UART1 ///////////////////////////////////////////////////////////////////////
  }
 else
  {
   return(0);
  } 
// 
 return(1);
}


////////////////////////////////////////////////////////////////////////////////
// Enable UART0/1 Tx interrupt, Set UART0/1 Tx Interrupt pending bit ///////////
////////////////////////////////////////////////////////////////////////////////

void UARTTxIntOn(int channel)
{
 if(channel == 1) 
  {
/* Enable Interrupt, Set pending Bit for UART1 */
   Enable_Int(nUART1_TX_INT);		/* INTMASK &= ~(1<<(n)) */
#ifdef _S3C4530_ 	
   if(is_s3c4530(syscfg_pd_id)) {	// 
     U32 stat;				// 
     UCON1 |= UCON_TFRST;		// 
     stat   = USTAT1;			// 
   }
#endif     							/* _S3C4530_ */
  }
 else 
 if(channel == 0) 
  {
   Enable_Int(nUART0_TX_INT);
#ifdef _S3C4530_ 	
   if(is_s3c4530(syscfg_pd_id)) {	// 
     U32 stat;				// 
     UCON0 |= UCON_TFRST;		// 
     stat   = USTAT0;			// 
   } 
#endif     							/* _S3C4530_ */
  }
}

////////////////////////////////////////////////////////////////////////////////
// Enable UART0/1 Rx/Status Interrupt //////////////////////////////////////////
////////////////////////////////////////////////////////////////////////////////

void UARTRxIntOn(int channel)
{
 if(channel == 1) 
  {
   Enable_Int(nUART1_RX_ERR_INT);
#ifdef _S3C4530_ 	
   if(is_s3c4530(syscfg_pd_id)) {
     U32 stat;	
     UCON1 |= UCON_RFRST;	
     stat   = USTAT1;
   }
#endif     							/* _S3C4530_ */ 
  }
 else 
 if(channel == 0)
  {
   Enable_Int(nUART0_RX_ERR_INT);
#ifdef _S3C4530_ 	
   if(is_s3c4530(syscfg_pd_id)) {
     U32 stat;
     UCON0 |= UCON_RFRST;
     stat   = USTAT0;
   }
#endif     							/* _S3C4530_ */	
  }
}

////////////////////////////////////////////////////////////////////////////////
// Diasable UART0/1 Tx Interrupt, Clear UART0/1 Tx Interrupt pending bit ///////
////////////////////////////////////////////////////////////////////////////////

void UARTTxIntOff(int channel)
{
 if(channel == 1) 
  {
   Disable_Int(nUART1_TX_INT);
   TxQ[1].wptr = 0;			// UART1 Tx Queue write pointer 	
   TxQ[1].rptr = 0;			// UART1 Tx Queue read pointer 
   TxQ[1].cnt  = 0;			// UART1 Tx Queue bytes count
  }
 else 
 if(channel == 0)
  {
/* Disable Interrupt */
   Disable_Int(nUART0_TX_INT);
   TxQ[0].wptr = 0;			// UART0 Tx Queue write pointer 	
   TxQ[0].rptr = 0;			// UART0 Tx Queue read pointer 
   TxQ[0].cnt  = 0;			// UART0 Tx Queue bytes count
  }
}

////////////////////////////////////////////////////////////////////////////////
// Disable UART0/1 Rx&Status Interrupt /////////////////////////////////////////
////////////////////////////////////////////////////////////////////////////////


void UARTRxIntOff(int channel)
{
 if(channel == 1) 
  {
// UART0 ///////////////////////////////////////////////////////////////////////
   Disable_Int(nUART1_RX_ERR_INT);	// disable UART1 rx & error int
   RxQ[1].wptr = 0;			// UART1 Rx Queue write pointer 	
   RxQ[1].rptr = 0;			// UART1 Rx Queue read pointer 
   RxQ[1].cnt  = 0;			// UART1 Rx Queue bytes count
 }
else 
if(channel == 0) 
 {
  Disable_Int(nUART0_RX_ERR_INT);
  RxQ[0].wptr = 0;			// UART0 Rx Queue write pointer 	
  RxQ[0].rptr = 0;			// UART0 Rx Queue read pointer 
  RxQ[0].cnt  = 0;			// UART0 Rx Queue bytes count
 }
}



////////////////////////////////////////////////////////////////////////////////
// read byte in interrupt mode /////////////////////////////////////////////////
////////////////////////////////////////////////////////////////////////////////

int i_getc(int channel, U8 * ch)
{
 if(channel < 0 || channel > 1 || ch == 0)
  {
   return(0);
  } 

 if(RxQ[channel].cnt == 0) 
  {						// Rx buffer empty	
   return(0);					// read byte failure
  }


 if(RxQ[channel].rptr >= MAXEVENT) {
   RxQ[channel].rptr = 0; 			// loop back
 }
 *ch = RxQ[channel].buff[RxQ[channel].rptr];  // read byte from Rx buffer 
 RxQ[channel].buff[RxQ[channel].rptr] = 0xFF;
 RxQ[channel].rptr++;

// 
 Disable_Int(channel ? nUART1_RX_ERR_INT : nUART0_RX_ERR_INT); 
 RxQ[channel].cnt--;
 Enable_Int (channel ? nUART1_RX_ERR_INT : nUART0_RX_ERR_INT); 
 return(1);					// 
}

////////////////////////////////////////////////////////////////////////////////
// write byte in interrupt mode ////////////////////////////////////////////////
////////////////////////////////////////////////////////////////////////////////


int i_putc(int channel, U8 ch)
{
 U32 stat;				// status registyer content 
 if(channel < 0 || channel > 1){
   return(0);
 } 

// 
 if(TxQ[channel].cnt >= MAXEVENT) {
// tx queue buffer full ////////////////////////////////////////////////////////
   return(0);
 }

#ifdef _S3C4530_				
// compiled with s3c4530 support ///////////////////////////////////////////////
 if(is_s3c4530(syscfg_pd_id)) {
   if(channel == 0) {		// read s3c4530 UART0 
     stat = USTAT0;		// status  register
   } else {	
     stat = USTAT1;		// s3c4530 UART1 status register
   }
   stat &= USTAT_THE;		// transmit holding register is empty
 } else {
// compiled with s3c4530 & CPU 4510 
   if(channel == 0) {		// read s3c4510 UART 0 status register
     stat = UARTSTAT0;	
   } else {
     stat = UARTSTAT1;		// read s3c4510 UART 1 status register
   }
   stat &= UARTSTAT_TXB_EMPTY;
 }
#else 
   if(channel == 0) {		// read s3c4510 UART 0 status register
     stat = UARTSTAT0;	
   } else {
     stat = UARTSTAT1;		// read s3c4510 UART 1 status register
   }
   stat &= UARTSTAT_TXB_EMPTY;
#endif								/* _S3C4530_ */

 if(stat != 0 && TxQ[channel].cnt == 0) {
   if(channel == 0) { 		
     UARTTXH0 = (U32) ch; 	// write character to UART0 holding register 
   } else { 
     UARTTXH1 = (U32) ch; 	// write character to UART1 holding register
   }
   return(1);
 }

 if(TxQ[channel].wptr >= MAXEVENT) {
   TxQ[channel].wptr = 0;
 }
// write character to buffer /////////////////////////////////////////////////// 
 TxQ[channel].buff[TxQ[channel].wptr++] = ch;

// 
 Disable_Int(channel ? nUART1_TX_INT : nUART0_TX_INT); 
 TxQ[channel].cnt++;			// tx bytes 
 Enable_Int (channel ? nUART1_TX_INT : nUART0_TX_INT); 
 return(1);
}

////////////////////////////////////////////////////////////////////////////////
// UART Interrupt put string ///////////////////////////////////////////////////
////////////////////////////////////////////////////////////////////////////////


int  i_puts(int channel, char * str)
{    
 if(channel < 0 || channel > 1 || str == 0)
  {
   return(0);
  } 

 while(*str != '\0')
  { 
/* Call i_putc to print each character.  */
   while(i_putc(channel, *str) == 0)
    {
     ;
    }
   str++;
  }

//  
 return(1);
}

////////////////////////////////////////////////////////////////////////////////
////////////////////////////////////////////////////////////////////////////////
////////////////////////////////////////////////////////////////////////////////


int i_flush(int channel)
{
 if(channel < 0 || channel > 1)
  {
   return(0);
  } 

// 
 while(TxQ[channel].cnt > 0) 
  {
   ;
  }
//
 return(1);
} 


////////////////////////////////////////////////////////////////////////////////
// put polling char from UART0/UART1 ///////////////////////////////////////////
////////////////////////////////////////////////////////////////////////////////



int put_char(int channel,char ch) 
{
 if(channel == 1) 
  {
#ifdef _S3C4530_
    if(is_s3c4530(syscfg_pd_id)) {
      while( (USTAT1 & USTAT_THE) == 0 ) {;}
    } else {
      while( (UARTSTAT1 & UARTSTAT_TXB_EMPTY) == 0 ) {;}
    }
#else 
      while( (UARTSTAT1 & UARTSTAT_TXB_EMPTY) == 0 ) {;}
#endif								/* _S3C4530_ */
   UARTTXH1 = (U32) ch;
   return(1);
  }
 else 
 if(channel == 0) 
  {
#ifdef _S3C4530_
    if(is_s3c4530(syscfg_pd_id)) {
      while( (USTAT0 & USTAT_THE) == 0 ) {;}
    } else {
      while( (UARTSTAT0 & UARTSTAT_TXB_EMPTY) == 0 ) {;}
    }
#else 
      while( (UARTSTAT0 & UARTSTAT_TXB_EMPTY) == 0 ) {;}
#endif								/* _S3C4530_ */
   UARTTXH0 = (U32) ch;
   return(1);
  } else {
    return(0);	
  }
}


////////////////////////////////////////////////////////////////////////////////
// get polling char from UART0/UART1 ///////////////////////////////////////////
////////////////////////////////////////////////////////////////////////////////


char get_char(int channel) 
{ 
 char ch;
 if(channel == 1) 
  { 
#ifdef _S3C4530_
    if(is_s3c4530(syscfg_pd_id)) {
      while( (USTAT1 & USTAT_RDV) == 0 ) {;}
    } else {
      while( (UARTSTAT1 & UARTSTAT_RCV_READY) == 0 ) {;}
    }
#else 
      while( (UARTSTAT1 & UARTSTAT_RCV_READY) == 0 ) {;}
#endif								/* _S3C4530_ */
   ch = UARTRXB1;
  }
 else 
 if(channel == 0)
  { 
#ifdef _S3C4530_
    if(is_s3c4530(syscfg_pd_id)) {
      while( (USTAT0 & USTAT_RDV) == 0 ) {;}
    } else {
      while( (UARTSTAT0 & UARTSTAT_RCV_READY) == 0 ) {;}
    }
#else 
      while( (UARTSTAT0 & UARTSTAT_RCV_READY) == 0 ) {;}
#endif								/* _S3C4530_ */
   ch = UARTRXB0;
  }
 else
  {
   ch = 0;
  }

// 
 return(ch);
}

int rcv_char(int ch, U8 * val)
{
 if(val == 0) {
   return(0); 
 }

 if(ch == 0) { 
#ifdef _S3C4530_
    if(is_s3c4530(syscfg_pd_id)) {
      if( (USTAT0 & USTAT_RDV) == 0 ) {
        return(0);
      }
    } else {
      if( (UARTSTAT0 & UARTSTAT_RCV_READY) == 0 ) {
        return(0);
      }
    }
#else 
      if( (UARTSTAT0 & UARTSTAT_RCV_READY) == 0 ) {
        return(0);
      }
#endif								/* _S3C4530_ */
   *val = (UARTRXB0&0xFF);
 } else
 if(ch == 1) { 
#ifdef _S3C4530_
    if(is_s3c4530(syscfg_pd_id)) {
      if( (USTAT1 & USTAT_RDV) == 0 ) {
        return(0);
      }
    } else {
      if( (UARTSTAT1 & UARTSTAT_RCV_READY) == 0 ) {
        return(0);
      }
    }
#else 
      if( (UARTSTAT1 & UARTSTAT_RCV_READY) == 0 ) {
        return(0);
      }
#endif								/* _S3C4530_ */
   *val = (UARTRXB1&0xFF);
 } else {
   return(0);  
 }
// 
 return(1); 
}

int  put_str(int ch, char * str)
{    
 if(ch < 0 || ch > 1 || str == 0) {
   return(0);
 } 
 while(*str != '\0')
  { 
/* Call i_putc to print each character.  */
   put_char(ch, *str);
   str++;
  }
//  
 return(1);
}

////////////////////////////////////////////////////////////////////////////////
// put char to console /////////////////////////////////////////////////////////
////////////////////////////////////////////////////////////////////////////////


void put_byte(char ch)
{
 put_char(CONSOLE, ch); 
}  

////////////////////////////////////////////////////////////////////////////////
// get char from console ///////////////////////////////////////////////////////
////////////////////////////////////////////////////////////////////////////////

char get_byte(void)
{
 char ch;
 ch = get_char(CONSOLE);
 CONSOLE_last = ch;
 return(ch);
}

////////////////////////////////////////////////////////////////////////////////
// get upper char from console /////////////////////////////////////////////////
////////////////////////////////////////////////////////////////////////////////

char get_upper(void)
{
 U8 ch;
 ch = get_byte();
 if(ch >= 'a' && ch <= 'z') {
// lowercase character => convert to upper case //////////////////////////
   ch = (ch - 'a') + 'A';		// 
 }
 CONSOLE_last = ch;
 put_byte(ch);
 return(ch);				// return character
}

char get_last(void)
{
 return(CONSOLE_last);
}

int kbd_hit(void) 
{
 if(CONSOLE == 1)
  {
#ifdef _S3C4530_
    if(is_s3c4530(syscfg_pd_id)) {
      if( (USTAT1 & USTAT_RDV) ) {
        return(1);
      }
    } else {
      if( (UARTSTAT1 & UARTSTAT_RCV_READY) ) {
        return(1);
      }
    }
#else 
      if( (UARTSTAT1 & UARTSTAT_RCV_READY) ) {
        return(1);
      }
#endif								/* _S3C4530_ */ 
  } 
 else
 if(CONSOLE == 0)
  {
#ifdef _S3C4530_
    if(is_s3c4530(syscfg_pd_id)) {
      if( (USTAT0 & USTAT_RDV) ) {
        return(1);
      }
    } else {
      if( (UARTSTAT0 & UARTSTAT_RCV_READY) ) {
        return(1);
      }
    }
#else 
      if( (UARTSTAT0 & UARTSTAT_RCV_READY) ) {
        return(1);
      }
#endif								/* _S3C4530_ */
  }  
 return(0);
}


////////////////////////////////////////////////////////////////////////////////
// UART0/UART1 Int LoopBack test ///////////////////////////////////////////////
////////////////////////////////////////////////////////////////////////////////
struct UARTERRBUF			// uart error recv bytes buffer
{
 U32 txseq;				// transmit sequence number 
 U32 rxseq;				// receive sequence number 
 U8 wait;				// waited value
 U8 recv;				// received 
}; 

struct UARTERRBUF uart_err_buf[32];
int uart_err_cnt = 0;

int put_uart_err(U32 ts, U32 rs, U8 w, U8 r)
{
 if(uart_err_cnt < 0 || uart_err_cnt >= 32) {
   return(0);				// out off error buffer 
 } else {
   struct UARTERRBUF * ui = uart_err_buf + uart_err_cnt++;
   ui->txseq  = ts;			// transmit sequence number 
   ui->rxseq  = rs;			// receive sequence number 
   ui->wait = w;			// waited byte value
   ui->recv = r;			// received byte value 
   return(1);
 }
}  

int print_uart_err(void)
{
 int en = 0;
 while(en < uart_err_cnt) {
   struct UARTERRBUF * ui = uart_err_buf + en++;
   Print("\nRx error:sequence num=[tx %d rx %d] wait=[0x%02x] received=[0x%02x]", 
    ui->txseq	,			// tx sequence number
    ui->rxseq	,			// rx sequence number
    ui->wait,				// waited byte 
    ui->recv				// received byte
   );
 } 
 uart_err_cnt = 0;
 return(1);
}

////////////////////////////////////////////////////////////////////////////////
// UART 0 & 1 internal loopback test ///////////////////////////////////////////
////////////////////////////////////////////////////////////////////////////////

int UARTIntLoopBack(void)
{
 int txcount[2] = {0, 0}; 		// tx bytes 
 int rxcount[2] = {0, 0}; 		// tx bytes 
 int errcount[2]= {0, 0};		// errors
 U32 ascii[2]   = {0, 0};		// sended byte
 U32 wait[2]    = {0, 0};		// next waited byte
 int more = 0;				// continue flag
 int rvl = 1;				// return code 
 U8 ch; 				// received character
 int cn;				// channel number

// 
 TimerReset(1);					// reset timer 0
 TimerInit (1,(ONE_SECOND/TICKS_PER_SECOND));	// init timer  0
 TimerStart(1);					// start timer 0

 Print("\nUART 0&1 Internal Loopback Test"); 

 for(cn = 0; cn < 2; ++cn)
  {
   UARTSetLoopBack(cn, 1);		// set loop back mode
   UARTRxIntOn(0); 			// enable UART cn Rx Interrupt
   UARTTxIntOn(0);			// enable UART cn Tx Interrupt
   more |= (1 << cn);			// set channel more flag 
  }
 Enable_Int(nGLOBAL_INT);		// enable global int 

 while(more)
  {
   for(cn = 0; cn < 2; ++cn) 		// 
    { 
// UART channel [cn] transmit loop /////////////////////////////////////////////
     if(ascii[cn] < 1024 * 256)
      {
       while(ascii[cn] < 1024 * 256)
        {
         if(i_putc(cn, ascii[cn]) == 0)
          {				// Tx Queue full
           break;				
          } 
	 txcount[cn]++;			// tx bytes 
         ascii[cn]++; 			// next sended byte
        } 
      }
     else 
     if(TxQ[cn].cnt == 0)
      {
// all test bytes are sended ///////////////////////////////////////////////////
       more &= ~(1 << cn);		// clear channel more flag
       UARTWaitTxComp(cn);		// wait last byte transmit completition
       TimerWaitMs(1, 100);		//
      }
    }

   for(cn = 0; cn < 2; ++cn) 
    {
     while(i_getc(cn, & ch) != 0)
      {
       rxcount[cn]++;			// rx bytes 
       if(ch != (U8) wait[cn])
        {
// received byte not equal waited //////////////////////////////////////////////
         put_uart_err(txcount[cn], rxcount[cn], wait[cn], ch); // 
         rvl = 0;			// clear return code 
         wait[cn] = ch;			// reset wait byte
         errcount[cn]++;		// rx errors
        } 
       wait[cn]++;			// next waited byte
      }
    }
  } 

 for(cn = 0; cn < 2; ++cn)
  {
   UARTTxIntOff(cn);			// disable tx int
   UARTRxIntOff(cn);			// disable rx int 
   UARTSetLoopBack(cn,0);		// clear loopback mode 	
  }
  
 TimerStop(1);				// reset timer 0
 TimerReset(1);

 Print("\nTx bytes=%d,%d, Rx bytes=%d,%d Errors=%d,%d",
   txcount[0], txcount[1], 
   rxcount[0], rxcount[1],
   errcount[0], errcount[1]
 );
 print_uart_err(); 			// 
 return(rvl);
}

////////////////////////////////////////////////////////////////////////////////
// UART External Loopback test /////////////////////////////////////////////////
////////////////////////////////////////////////////////////////////////////////

int UARTExtLoopBackInt(int chan)
{
 int rvl = 1;				// return code 
 int cn;				// channel number 

// 
 TimerReset(1);					// reset timer 0
 TimerInit (1,(ONE_SECOND/TICKS_PER_SECOND));	// init timer  0
 TimerStart(1);					// start timer 0

 for(cn = 0; cn < 2; ++cn) 
  { 
   U32 ascii = 0;			// sended byte
   U32 wait = 0;			// next waited byte
   int txcount = 0; 			// tx bytes 
   int rxcount = 0; 			// tx bytes 
   int errcount = 0;			// errors
   int more = 1;			// exit flag
   U8 ch; 

   if((cn == chan || chan == -1) == 0) 
    {
      continue;
    }  

   Print("\nUART%d External Loopback Test", cn); 
   UARTRxIntOn(cn); 			// enable UART Rx Interrupt
   UARTTxIntOn(cn);			// enable UART Tx Interrupt
   Enable_Int(nGLOBAL_INT);		// enable global int 

   while(more) 
    {
     if(ascii < 1024 * 256)
      {
// send bytes 
       while(ascii < 1024 * 256)
        {
         if(i_putc(cn, ascii) == 0)
          {				// Tx Queue full
           break;				
          } 
	 txcount++;
         ascii++; 			// next sended byte
        } 
      }
     else 
     if(TxQ[cn].cnt == 0)
      {
// all test bytes are sended 
       more = 0;
       UARTWaitTxComp(cn);		// wait last byte transmit completition
       TimerWaitMs(1, 100);	
      }

     while(i_getc(cn, & ch) != 0)
      {
       rxcount++;				// rx bytes 
       if(ch != (U8) wait)
        {
// received byte not equal waited //////////////////////////////////////////////
         if(cn == CONSOLE) {		// write error to report buffer
           put_uart_err(txcount, rxcount, wait, ch); // 
         } else {
           Print("\nRx error sended=[0x%02x] wait=[0x%02x] receive=[0x%02x]", 
            (ascii & 0xFF), wait, ch);
         }
         rvl = 0;			// clear return code 
         wait = ch;			// reset wait byte
         errcount++;			// rx errors
        } 
       wait++;				// next waited byte
      }
    }

   UARTRxIntOff(cn); 			// enable UART Rx Interrupt
   UARTTxIntOff(cn);			// enable UART Tx Interrupt
   Print("\nTx bytes=%d, Rx bytes=%d, Errors=%d", txcount, rxcount, errcount);
   if(cn == CONSOLE) {			// print error report 
     print_uart_err(); 			// 
   }
  }

 TimerStop(1);				// reset timer 0
 TimerReset(1);
 return(rvl);
}

////////////////////////////////////////////////////////////////////////////////
// loopback test interrupt mode (original) /////////////////////////////////////
////////////////////////////////////////////////////////////////////////////////


U32 UARTLoopBackInt(int channel)
{
 U32 txcount  = 0;			// transmit bytes 
 U32 rxcount  = 0;			// receive bytes 
 U32 errcount = 0;			// Rx errors (received byte not equal waited)
 int rvl = 1;				// return code
 int more = 1;				// continue flag 
 int ascii;				// sended byte
 U32 msstart;				// start millisecond 
 U32 mscur;				// current millisecond
 U32 mswait=0;
 TIME tm;				// TIME struct 
 U8 wait;				// waited byte
 U8 ch;					// received byte

 if(channel < 0 || channel > 1)
  {					// invalid channel number
   return(0);
  } 

// ptint test title ////////////////////////////////////////////////////////////
 Print("\n\nUART%d internal Loopback test (interrupt mode)...", channel);
 if(channel != CONSOLE)
  {
   Print("\nTo break test press any key...");
   while(kbd_hit()) {			// clear console input 
     ch = get_byte();
    }
  }

// wait UART transmit complete & set loopback mode ////////////////////////////
 UARTWaitTxComp(channel);		// wait UART0 transmit complete
 UARTSetLoopBack(channel, 1);		// set loop back mode

 TimerReset(1);					// reset timer 0
 TimerInit (1,(ONE_SECOND/TICKS_PER_SECOND));	// init timer  0
 TimerStart(1);					// start timer 0
//
 UARTRxIntOn(channel); 			// enable UART Rx Interrupt
 UARTTxIntOn(channel);			// enable UART Tx Interrupt
 Enable_Int(nGLOBAL_INT);		// enable global int 

// 
 GetSysTime(1, & tm);	
 msstart = tm.tm_ms + (tm.tm_sec + tm.tm_min * 60) * 1000;

 wait = 0;				// start waited byte 
 ascii = 0;				// start sended byte 

 while(more)
  {
//
   if(ascii < 1024 * 256)
    {
     while(ascii < 1024 * 256)
      {
       if(i_putc(channel, ascii) == 0)
        {				// Tx Queue full
         break;				
        } 

       txcount++; 			// calulate tx bytes
       ascii++; 			// next sended byte
      }
    }
   else
    {
     if(TxQ[channel].cnt == 0)
      {
// Tx queue empty => clear more flag /////////////////////////////////////////// 
       more = 0;
       UARTWaitTxComp(channel);		// wait last byte transmit completition
       TimerWaitMs(1, 100);	
      }
    } 


// receive & check bytes ///////////////////////////////////////////////////////

   while(i_getc(channel, & ch) != 0)
    {
     rxcount++;				// rx bytes 
     if(ch != wait)
      {
// received byte not equal waited //////////////////////////////////////////////
       if(channel == CONSOLE) {		// write error to report buffer
         put_uart_err(txcount, rxcount, wait, ch); // 
       } else {
         Print("\nRx error sended=[0x%02x] wait=[0x%02x] receive=[0x%02x]", 
          (ascii & 0xFF), wait, ch);
       }
       rvl = 0;				// clear return code 
       wait = ch;			// reset wait byte
       errcount++;			// rx errors
      } 
     wait++;				// next waited byte
    }

   if(channel == CONSOLE)
    {
     continue;
    } 

// 
   if(kbd_hit())
    {
/////////////////////////////////////////////////////////////////////////////////
// test breaked from console ////////////////////////////////////////////////////
/////////////////////////////////////////////////////////////////////////////////
     ch = get_byte();			// 

     more = 1;
     while(more)
      {
       if(TxQ[channel].cnt == 0)
        {
         more = 0;
         UARTWaitTxComp(channel);	// wait last byte transmit completition
         TimerWaitMs(1,100);
        }

//        
       while(i_getc(channel, & ch) != 0)
        {
         rxcount++;			// rx bytes 
         if(ch != wait)
          {
// received byte not equal waited //////////////////////////////////////////////
           if(channel != 0)
            {
             Print("\nRx error sended=[0x%02x] wait=[0x%02x] receive=[0x%02x]", (ascii & 0xFF), wait, ch);
            }
           rvl = 0;			// clear return code 
           wait = ch;			// reset wait byte
           errcount++;			// rx errors
          } 
         wait++;			// next waited byte
        }
      }
    }
  }

//
 GetSysTime(1, & tm);
 mscur = tm.tm_ms + (tm.tm_sec + tm.tm_min * 60) * 1000;
 mscur -= msstart + mswait;

 TimerStop(1);				// reset timer 0
 TimerReset(1);

 UARTRxIntOff(channel); 		// disable UART Rx Interrupt
 UARTTxIntOff(channel);			// disable UART Tx Interrupt
 UARTSetLoopBack(channel, 0);		// clear loopback

 if(txcount != rxcount || errcount != 0)
  {
   rvl = 0;
  } 

// print test statistics ///////////////////////////////////////////////////////
 Print("\nTransmit bytes = [%d]", txcount);	// tx bytes 
 Print("\nBytes sec      = [%d]",(txcount * 1000 / mscur));
 Print("\nReceive bytes  = [%d]", rxcount);	// rx bytes 
 Print("\nReceive errors = [%d]", errcount);	// rx errors
// 
 Print("\nOverrun errors = [%d]", UART_ERR[channel].overrun);	
 Print("\nParity  errors = [%d]", UART_ERR[channel].parity );
 Print("\nFrame   errors = [%d]", UART_ERR[channel].frame  );
 Print("\nBreak          = [%d]", UART_ERR[channel].brdet  );
// clear error statistics //////////////////////////////////////////////////////
 UART_ERR[channel].overrun = 0;		// overrun errors
 UART_ERR[channel].parity  = 0;		// parity  errors
 UART_ERR[channel].frame   = 0;		// frame   errors
 UART_ERR[channel].brdet   = 0;		// 
// 
 if(rvl == 0 && channel == CONSOLE) {	
   print_uart_err();			// print uart rx error buffer
 } 

// 
 if(rvl == 1) 
  {
   Print("\nOk.\n");
  }
 else 
  {
   Print("\nFail.\n");
  } 
//
 return(rvl);
}

////////////////////////////////////////////////////////////////////////////////
// UART Loop back test in pooling mode /////////////////////////////////////////
////////////////////////////////////////////////////////////////////////////////

U32 UARTAutoLoopBack(int channel)
{
 U32 rvl[2] = {1, 1};
 int cn;

 TimerReset(1);					// reset timer 0
 TimerInit (1,(ONE_SECOND/TICKS_PER_SECOND));	// init timer  0
 TimerStart(1);					// start timer 0
 
 for(cn = 0; cn < 2; ++cn) 
  {
   U32 ascii;				// sequence number
   U32 cnt;				// send bytes 
   U32 err;				// error count 
   U8 wait;				// waited rx byte 
   U8  ch;				// received byte 

   if((cn == channel || channel == -1) == 0)
    {
     continue;
    } 

//  UART Lop back test /////////////////////////////////////////////////////////
   Print("\nUART%d internal Loopback test (polling mode)...", cn);
   UARTRxIntOff(cn); 			/* disable UART0 Rx Interrupt */
   UARTTxIntOff(cn);			/* disable UART0 Tx Interrupt */ 
   UARTWaitTxComp(cn);			/* wait tx comp */
   UARTSetLoopBack(cn,1);		/* Set Loopback test mode */
   cnt = UARTGetSpeed(cn);		/* tx bytes count */
   err = 0;				/* error count */

   for(ascii = 0, wait = 0; cnt-- > 0; ascii++) 
    {
     put_char(cn, ascii);
     ch = get_char(cn);
     if(ch != (U8) wait) 
      {
       err += 1;
       rvl[cn] = 0;
       wait = ch; 	
       put_uart_err(ascii,ascii,(U8) ascii, ch);
      }
     wait++; 
    }
   UARTSetLoopBack(cn,0);		/* clear loopback mode */
   if(rvl[cn] == 1) 
    {
     Print("\nOk.\n");
    }
   else 
    {
     Print("\nFail. Tx bytes=[%d], Rx bytes=[%d],Err count=[%d]\n", 
            ascii, ascii, err);
     print_uart_err();
    } 
  }

 if(rvl[0] == 1 && rvl[1] == 1) 
  {
   return(1);
  }
 else 
  {
   return(0);
  }
}


/////////////////////////////////////////////////////////////////////////////////
// UART External loop back test in interrupt mode ///////////////////////////////
/////////////////////////////////////////////////////////////////////////////////


void StringEcho(U32 channel)
{
 U8 ch;

 do 
  {
   Print("\n\nString echo for UART");
   if(channel == 0)
    {
     Print("0 interrupt test.");
    }
   else
   if(channel == 1)
    {
     Print("1 interrupt test.");
    }
   else
    {
     while(1)
      {
       Print("\nSelect UART [0/1]");
       channel = get_upper();
       if(channel >= '0' && channel <= '1')
        {
         channel -= '0';
         break;
        }
      }
    }

 //
   Print("\nInput characters will be echoed on Console");
   Print("\nTo escape, enter ESC key : ");

   UARTTxIntOn(channel);		// Enable UART Tx Interrupts
   UARTRxIntOn(channel);		// Enable UART Rx Interrupts 
   Enable_Int(nGLOBAL_INT);

   do 
    {
     while( i_getc(channel,& ch) == 0); 
     while(!i_putc(channel,ch))
      {
      }
    }
    while(ch != KEY_CR && ch != KEY_ESC);

   UARTTxIntOff(channel);		// Enable UART Tx Interrupts
   UARTRxIntOff(channel);		// Enable UART Rx Interrupts 
  }
 while(ch != KEY_ESC);

// 
 Print("\n");
 UARTTxIntOff(channel);			// disable UART Tx Interrupts
 UARTRxIntOff(channel);			// disable UART Rx Interrupts 
}

#ifdef _KS32C50100_
/////////////////////////////////////////////////////////////////////////////////
// Print UARTLCON0, UARTLCON1 ///////////////////////////////////////////////////
/////////////////////////////////////////////////////////////////////////////////

static int PrintUARTLCON(int num)
{
 U32 uartlcon;				// UARTLCON register value
 if(num == 0)
  {
   uartlcon = UARTLCON0;		// UARTLCON0
  }
 else
 if(num == 1)
  {
   uartlcon = UARTLCON1;		// UARTLCON1
  }
 else
  {
   return(0);
  } 

// 
 Print("\nUARTLCON%d (UART%d line control register) = 0x%08x", 
   num		, 			// UART number 
   num		, 			// UART number 
   uartlcon				// register value
  );

// 1:0 Word length (WL) ////////////////////////////////////////////////////////
 Print("\n1:0 Word length (WL)........");
 switch( (uartlcon & UARTLCON_WL) )	// Word Length mask 
  {
   case UARTLCON_WL5 : Print("00 = 5 bits"); break;	// 5 bits
   case UARTLCON_WL6 : Print("01 = 6 bits"); break;	// 6 bits
   case UARTLCON_WL7 : Print("10 = 7 bits"); break;	// 7 bits
   case UARTLCON_WL8 : Print("11 = 8 bits"); break;	// 8 bits
   default           : break;
  }

// [2] Number of Stop bits /////////////////////////////////////////////////////
 Print("\n2   Number of Stop bits.....%s",
  (uartlcon & UARTLCON_STOP_2)	?
  "1(two Stop bits per frame)"	:	
  "0(one Stop bit per frame)"
  );

// [5:3] Parity mode (PMD) /////////////////////////////////////////////////////
 Print("\n5:3 Parity mode (PMD).......");
 switch( (uartlcon & UARTLCON_PMD) )
  {
   case UARTLCON_PMD_NO   :  Print("000(no parity)"); break;
   case UARTLCON_PMD_ODD  :  Print("100(odd parity)"); break;
   case UARTLCON_PMD_EVEN :  Print("101(even parity)"); break;
   case UARTLCON_PMD_CHK1 :  Print("110(parity is forced,checked as a 1)"); break;
   case UARTLCON_PMD_CHK0 :  Print("111(parity is forced,checked as a 0)"); break;
   default 		  :
    break;
  }

// [6] Serial Clock Selection //////////////////////////////////////////////////
 Print("\n6   Serial Clock Selection..%s",
   (uartlcon & UARTLCON_UCLK)	?	/* for external UART clock */
   "1(External UCLK)"		:
   "0(Internal MCLK)"
  );
 
// [7] Infra-red mode //////////////////////////////////////////////////////////
 Print("\n7   Infra-red mode..........%s",
  (uartlcon & UARTLCON_INFRA_RED) ?
   "1(Infra-red Tx/Rx mode)"	:
   "0(Normal mode operation)"
  );

//
 return(1);
}
#endif

#ifdef _KS32C50100_
/////////////////////////////////////////////////////////////////////////////////
// Print UART0/UART1 Control Register ///////////////////////////////////////////
/////////////////////////////////////////////////////////////////////////////////
static int PrintUARTCONT(int num)
{
 U32 uartcont;			
 if(num == 0) {			
   uartcont = UARTCONT0;
 } else
 if(num == 1) { 
   uartcont = UARTCONT1;
 } else { 
   return(0);
 }     

// 
 Print("\nUARTCONT%d UART%d Control Register = 0x%08x", 
   num		,
   num		,
   uartcont
 );

// [1:0] Receive mode (RxM) ////////////////////////////////////////////////////
 Print("\n1:0  Receive mode (RxM).................");
 switch(uartcont & UARTCON_RXM) 	/* receive mode mask */	
  {
   case UARTCON_RXM_OFF      : Print("00(receive disabled)"); break;
   case UARTCON_RXM_INTREQ   : Print("01(status int request)"); break;
   case UARTCON_RXM_GDMA0REQ : Print("10(GDMA channel 0 req.)"); break;
   case UARTCON_RXM_GDMA1REQ : Print("11(GDMA channel 1 req.)"); break;
   default                   : break; 
  }

// [2]   Rx status interrupt enable (RxSI) /////////////////////////////////////
 Print("\n2    Rx status interrupt enable (RxSI)..%s",
  (uartcont & UARTCON_RXSTAT_INT) ?	
   "1(enable)"		:	
   "0(disable)"
  );

// [4:3] Transmit mode (TxM) ///////////////////////////////////////////////////
 Print("\n4:3  Transmit mode (TxM)................");
 switch(uartcont & UARTCON_TXM)	
  {
   case UARTCON_TXM_OFF      : Print("00(transmit disabled)"); break;
   case UARTCON_TXM_INTREQ   : Print("01(interrupt request)"); break;
   case UARTCON_TXM_GDMA0REQ : Print("10(GDMA channel 0 req.)"); break;
   case UARTCON_TXM_GDMA1REQ : Print("11(GDMA channel 1 req.)"); break;
   default :
    break;
  }  

// [5]   Data set ready (DSR) //////////////////////////////////////////////////
 Print("\n5    Data set ready (DSR)...............%s",
  (uartcont & UARTCON_DSR)	    ? 	/* Assert DSR output(nUADSR)*/
   "1"	:
   "0"
  );
  
// [6]   Send break (SBK) //////////////////////////////////////////////////////
 Print("\n6    Send break (SBK)...................%s",
  (uartcont & UARTCON_SEND_BREAK) ?
   "1"	:	
   "0"
  );

// [7]   Loop-back mode (LPB) //////////////////////////////////////////////////
 Print("\n7    Loop-back mode (LPB)...............%s",
  (uartcont & UARTCON_LOOPBACK) ?	/* for test only */
   "1"	:
   "0"
  );

//
 return(1);
}
#endif

/////////////////////////////////////////////////////////////////////////////////
// print UART0,1 status register S3C4530 ////////////////////////////////////////
/////////////////////////////////////////////////////////////////////////////////

// print channel register bit (bits) title //////////////////////////////////////
void PrintMBIT(char * mes)
{
 Print("\n");				// move to new line 
 if(mes != 0)
  {
   int pos = 0;		
   while(*(mes+pos++) != '\0') {;}	// calc string length
   Print("%s", mes);			// print bit (bits) name  
   while(pos++ < 40) {Print(".");} 	// print right padding
  }
}

////////////////////////////////////////////////////////////////////////////////
// print channel register bit value 0 or 1 /////////////////////////////////////
////////////////////////////////////////////////////////////////////////////////

void PrintUBIT(int ch, char * mes, U32 * val, U32 msk)
{
 PrintMBIT(mes); 			// print bit name
 
 if(val != 0)
  {
   int cnt;
   if(ch >= 0) {
// ch >= 0 => ch - register number ///////////////////////////////////////////// 
     Print("%-15d", (*(val+ch)&msk) ? 1 : 0);
   } else {
// ch < 0  => (-ch) - count of regs
     int len;  				// field length 
     int cn;				// channel number 
     cnt = -ch;				// channel count 
     len = 40 / cnt;			// field length 
     if(len > 15) { len = 15; }		// set field length to maximal value
     for(cn = 0; cn < cnt; ++cn) {
       Print("%-*d", len, (*(val+cn)&msk) ? 1 : 0);
     }
   }
  }
}

// print channel register bit value ///////////////////////////////////////////// 
void PrintSBIT(int ch, char * mes, U32 * st, U32 msk, char * v1, char * v0)
{
 int cn, cnt, len;			// channel number, count, field length
 PrintMBIT(mes);			// print register name
 if(st == 0)
  {
   return;
  }

 if(v1 == 0) { v1 = "1"; }		// set v1 to default value
 if(v0 == 0) { v0 = "0"; }  		// set v0 to default value 

 if(ch >= 0) {
    len = 15;
    cnt = ch + 1;
 } else {
    cnt = -ch;
    len = 40 / cnt;
    if(len > 15) { len = 15; }
 }

 for(cn = 0; cn < cnt; ++cn)
  {  
   if(cn == ch || ch < 0)
    {
     if( (*(st+cn) & msk) != 0)		// bit set to 1
      {
 	Print("%-*s", len, v1);		// print v1 for seted bit 
      }	 
     else
      {
	Print("%-*s", len, v0);		// print v0 for cleared bit
      } 
    }  
  }
}

// print channel register bit value ///////////////////////////////////////////// 
void PrintEBIT(int ch, char * mes, U32 * st, U32 msk)
{
 PrintSBIT(ch, mes, st, msk, "1(enable)", "0(disable)");
}

// print channel register bits value //////////////////////////////////////////// 
void PrintABIT(int ch, char * mes, U32 * st, U32 msk, ABIT * abit)
{
 int cn, cnt, len;
 PrintMBIT(mes);				// print bits name 
 if(st == 0 || abit == 0)
  {
   return;
  }

 if(ch >= 0) {
    cnt = ch + 1;
    len = 15;
 } else {
    cnt = -ch;
    len = 40 / cnt;
    if(len > 15) { len = 15; }
 }

 for(cn = 0; cn < cnt; ++cn) 
  {
   int vn, fnd;
   if( (cn == ch || ch < 0) == 0 ) {
     continue;
   }

   fnd = 0;
   for(vn = 0; (abit + vn)->str != NULL; ++vn) 
    {
     if( (*(st + cn) & msk) == (abit + vn)->val ) 
      {
       fnd = 1;
       Print("%-*s", len, (abit + vn)->str); 
       break;
      }
    }
   if(fnd == 0)
    {
     Print("%-*s", len, "(undef)"); 
    }
  }
}

#ifdef _S3C4530_
////////////////////////////////////////////////////////////////////////////////
// print UCON0/UCON1 S3C4530 UART Control register value ///////////////////////
////////////////////////////////////////////////////////////////////////////////
static int PrintUCONT(int ch)
{
 U32 cont[2];				//
 cont[0] = UCON0;			// UART0 control register 
 cont[1] = UCON1;			// UART1 control register

//
 if(ch >= 0 && ch <= 1) 
  {
   Print("\nUCON%d Control Register = 0x%08x", 
     ch, 				// channel number 
     cont[ch]				// channel control reg value
   );
  }   
 else 
  {
   Print("\nUCON 0&1 Control Registers = 0x%08x 0x%08x", cont[0], cont[1]);
  }   

// transmit mode 
 {
  ABIT tx_md[] =  { 			// transmit mode bit definition
   {"00(disable)"  , UCON_TMODE_OFF     },
   {"01(int req)"  , UCON_TMODE_INTREQ  },
   {"10(GDMA0 req)", UCON_TMODE_GDMA0REQ},
   {"11(GDMA1 req)", UCON_TMODE_GDMA1REQ},
    0
  };  

  PrintABIT(ch, "1:0   Transmit mode (TMODE)", cont, UCON_TMODE, tx_md);
 }


// receive mode 
 {
  ABIT rx_md[] = {			// receive mode bit definition 
   {"00(disable)"  , UCON_RMODE_OFF     },
   {"01(int req)",   UCON_RMODE_INTREQ  },
   {"10(GDMA0 req)", UCON_RMODE_GDMA0REQ},
   {"11(GDMA1 req)", UCON_RMODE_GDMA1REQ},
    0 
  };
 PrintABIT(ch, "3:2   Receive mode (RMODE)", cont, UCON_RMODE, rx_md);
 } 

// 
 PrintUBIT(ch, "4     Send Break (SBR)"             , cont, UCON_SBR  );
 PrintSBIT(ch, "5     Serial Clock Selection (UCLK)", cont, UCON_UCLK ,
   "1(UCLK)", "0(MCLK)"
 ); 
 PrintUBIT(ch, "6     Auto Baud Rate Detect (ABRD)" , cont, UCON_ABRD );
 PrintSBIT(ch, "7     Look-back mode (LOOPB)"       , cont, UCON_LOOPB,
  "1(loopback)", "0"
 ); 

// parity mode 
 {
 ABIT p_md[] = {			// parity mode bit definition 
   {"0xx(no)"     , UCON_PMD_NO  },
   {"100(odd)"    , UCON_PMD_ODD },
   {"101(even)"   , UCON_PMD_EVEN},
   {"110(1)"	  , UCON_PMD_CHK1},
   {"111(0)"      , UCON_PMD_CHK0},
    0
  };  
 PrintABIT(ch, "10:8  Parity mode (PMD)", cont, UCON_PMD, p_md);
 }

//
 PrintSBIT(ch, "11    Number of Stop bits (STB)", cont, UCON_STOP_2, "2", "1");

// word length 
 {
  ABIT wd_bt[] = {			// word length bit definition 
   {"00(5 bits)", UCON_WL5}, 
   {"01(6 bits)", UCON_WL6},
   {"10(7 bits)", UCON_WL7},
   {"11(8 bits)", UCON_WL8},
    0
  };
 PrintABIT(ch, "13:12 Word Length (WL)"           , cont, UCON_WL, wd_bt);
 }

//
 PrintSBIT(ch, "14    Infra-red mode"             , cont, UCON_IR, "1", "0");
 Print  ("\n15    Reserved");
 PrintEBIT(ch, "16    Transmit FIFO enable (TFEN)", cont, UCON_TFEN );
 PrintEBIT(ch, "17    Receive FIFO enable (RFEN)" , cont, UCON_RFEN );
 PrintUBIT(ch, "18    Transmit FIFO reset (TFRST)", cont, UCON_TFRST);
 PrintUBIT(ch, "19    Receive FIFO reset (RFRST)" , cont, UCON_RFRST);

//
 {
  ABIT tf_tl[] =  {
   {"00=30/32",UCON_TFTL_30},
   {"01=24/32",UCON_TFTL_24},
   {"10=16/32",UCON_TFTL_16},
   {"11=8/32" ,UCON_TFTL_8 },
    0
  };

 PrintABIT(ch, "21:20 Tx FIFO trigger level (TFTL)..", cont, UCON_TFTL, tf_tl);
 }

// receive fifo trigger level 
 {
 ABIT rf_tl[] =  {
   {"00 = 1/32" , UCON_RFTL_1 },
   {"01 = 8/32" , UCON_RFTL_8 },
   {"10 = 16/32", UCON_RFTL_16},
   {"11 = 24/32", UCON_RFTL_24},
    0
  };  
 PrintABIT(ch, "23:22 Rx FIFO trigger level (RFTL)..", cont, UCON_RFTL, rf_tl); 
 }

 PrintUBIT(ch, "24    Data Terminal Ready to pin (DTR)", cont, UCON_DTR);
 PrintUBIT(ch, "25    Request to Send to pin (RTS)....", cont, UCON_RTS);
 Print  ("\n27:26 Reserved");
 PrintUBIT(ch, "28    Hardware Flow Control En(HFEN)"  , cont, UCON_HFEN);
 PrintUBIT(ch, "29    Software Flow Control En(SFEN)"  , cont, UCON_SFEN);
 Print  ("\n31:30 Reserved");
 return(1);
}
#endif								/* _S3C4530_ */ 


/////////////////////////////////////////////////////////////////////////////////
// Print UART0/UART1 Status Register ////////////////////////////////////////////
/////////////////////////////////////////////////////////////////////////////////

#ifdef _KS32C50100_
static int PrintUARTSTAT(int num)
{
 U32 uartstat;
 if(num == 0)
  {
   uartstat = UARTSTAT0;
  } 
 else
 if(num == 1)
  {
   uartstat = UARTSTAT1;
  }
 else
  {
   return(0);
  } 

 Print("\nUARTSTAT%d UART%d Status Register = 0x%08x",
   num		,
   num 		,
   uartstat
  );

// [0] Overrun error ///////////////////////////////////////////////////////////
 Print("\n0 Overrun error..............%d",
   ((uartstat & UARTSTAT_OVERRUN) != 0)	? 1 : 0		/* overrun error */
  );

// [1] Parity error ////////////////////////////////////////////////////////////
 Print("\n1 Parity error...............%d",
  ((uartstat & UARTSTAT_PARITY) != 0) ? 1 : 0		/* parity error */
  );

// [2] Frame error /////////////////////////////////////////////////////////////
 Print("\n2 Frame error................%d",
   ((uartstat & UARTSTAT_FRAME) != 0) ? 1 : 0		/* frame error */
  );

// [3] Break interrupt /////////////////////////////////////////////////////////
 Print("\n3 Break interrupt............%d",
   ((uartstat & UARTSTAT_BREAK) != 0) ? 1 : 0		/* break interrupt */
  );

// [4] Data terminal ready (DTR) ///////////////////////////////////////////////
 Print("\n4 Data terminal ready (DTR)..%d",
   ((uartstat & UARTSTAT_DTR_LOW) != 0)	? 1 : 0		/* data terminal ready */
  );

// [5] Receive data ready //////////////////////////////////////////////////////
 Print("\n5 Receive data ready.........%d",
   ((uartstat & UARTSTAT_RCV_READY) != 0) ? 1 : 0	/* receive data ready */ 
  );

// [6] Tx Buffer register empty ////////////////////////////////////////////////
 Print("\n6 Tx Buffer register empty...%d",
   ((uartstat & UARTSTAT_TXB_EMPTY) != 0) ? 1 : 0	/* tx buffer empty */
  );

// [7] Transmit complete (TC) //////////////////////////////////////////////////
 Print("\n7 Transmit complete (TC).....%d",
   ((uartstat & UARTSTAT_TX_COMPLET) != 0) ? 1 : 0	/* transmit complete */
  );

// 
 return(1);
}
#endif

#ifdef _S3C4530_
// print status registers 
static void PrintUSTAT(int ch)
{
 U32 stat[2];
 stat[0] = USTAT0;
 stat[1] = USTAT1;

// status register content 
 if(ch >= 0 && ch <= 1) 
  {
   Print("UART%d status register=0x%08x" , ch, stat[ch]); 
  }
 else 
  {
   Print("UART 0&1 status registers=0x%08x,0x%08x" , stat[0], stat[1]); 
  } 


// 
 PrintUBIT(ch,"0  Receive Data Valid (RDV)"     , stat, USTAT_RDV);
 PrintUBIT(ch,"1  Break Signal Detected (BSD)"  , stat, USTAT_BSD);
 PrintUBIT(ch,"2  Frame Error (FER)"            , stat, USTAT_FER);
 PrintUBIT(ch,"3  Parity Error (PER)"           , stat, USTAT_PER);
 PrintUBIT(ch,"4  Overrun Error (OER)"          , stat, USTAT_OER);
 PrintUBIT(ch,"5  Control Character Detect CCD)", stat, USTAT_CCD);
 PrintUBIT(ch,"6  Data carrier Detect (DCD)"    , stat, USTAT_DCD);
 PrintUBIT(ch,"7  Rx FIFO trigger level(RFREA)" , stat, USTAT_RFREA);
 PrintUBIT(ch,"8  Receive FIFO empty (RFEMT)"   , stat, USTAT_RFEMT);
 PrintUBIT(ch,"9  Receive FIFO full (RFFUL)"    , stat, USTAT_RFFUL);
 PrintUBIT(ch,"10 Receive FIFO overrun (RFOV)"  , stat, USTAT_RFOV);
 PrintUBIT(ch,"11 Receiver in idle (RIDLE)"     , stat, USTAT_RIDLE);
 PrintUBIT(ch,"12 Rx Event time out (E_RxTO)"   , stat, USTAT_E_RxTO);
 Print  ("\n13 Reserved Not applicable");
 PrintUBIT(ch,"14 Data Set ready (DSR)"         , stat, USTAT_DSR);
 PrintUBIT(ch,"15 Clear To Send (CTS)"          , stat, USTAT_CTS);
 PrintUBIT(ch,"16 CTS Event occurred (E_CTS)"   , stat, USTAT_E_CTS);
 PrintUBIT(ch,"17 Transmit Complete (TC)"       , stat, USTAT_TC);
 PrintUBIT(ch,"18 Tx Holding Reg Empty (THE)"   , stat, USTAT_THE);
 PrintUBIT(ch,"19 Transmit FIFO Empty (TFEMT)"  , stat, USTAT_TFEMT);
 PrintUBIT(ch,"20 Transmit FIFO full (TFFUL)"   , stat, USTAT_TFFUL); 
 Print  ("\n31:21 Reserved");
}
#endif								/* _S3C4530_ */

#ifdef _S3C4530_
/////////////////////////////////////////////////////////////////////////////////
// print UART interrup enable register S3C4530 //////////////////////////////////
/////////////////////////////////////////////////////////////////////////////////
static void PrintUINTEN(int ch)
{
 U32 inten[2];
 inten[0] = UINTEN0;
 inten[1] = UINTEN1;

// prit interrupt register(s) content /////////////////////////////////////////// 
 if(ch >= 0 && ch <= 1) {
   Print("UART%d int enable register=0x%08x" , ch, inten[ch]); 
 } else {
   Print("UART 0&1 int enable registers=0x%08x,0x%08x", inten[0], inten[1]); 
 } 

//
 PrintUBIT(ch,"0  Receive Data Valid(RDVIE)"     , inten, UINTEN_RDVIE); 
 PrintUBIT(ch,"1  Break Signal Detected(BSDIE)"  , inten, UINTEN_BSDIE);
 PrintUBIT(ch,"2  Frame Error(FERIE)"            , inten, UINTEN_FERIE);
 PrintUBIT(ch,"3  Parity Error(PERIE)"           , inten, UINTEN_PERIE);
 PrintUBIT(ch,"4  Overrun Error(OERIE)"          , inten, UINTEN_OERIE);
 PrintUBIT(ch,"5  Control Char Detect(CCDIE)"    , inten, UINTEN_CCDIE);
 PrintUBIT(ch,"6  DCD High at check time(DCDLIE)", inten, UINTEN_DCDLIE);
 PrintUBIT(ch,"7  Rx FIFO trigger lev(RFREAIE)"  , inten, UINTEN_RFREAIE);
 Print  ("\n9:8 Reserved");
 PrintUBIT(ch,"10 Receive FIFO overrun(OVFFIE)"  , inten, UINTEN_OVFFIE);
 Print  ("\n11 Reserved");
 PrintUBIT(ch,"12 Rx Event timeout(E_RxTOIE)"    , inten, UINTEN_E_RxTOIE);
 Print  ("\n15:13 Reserved");
 PrintUBIT(ch,"16 CTS Event occurred(E_CTSIE)"   , inten, UINTEN_E_CTSIE);
 Print  ("\n17 Reserved");
 PrintUBIT(ch,"18 Tx Holding Reg Empty(THEIE)"   , inten, UINTEN_THEIE);
 Print  ("\n31:19 Reserved");
}
#endif								/* _S3C4530_ */

/////////////////////////////////////////////////////////////////////////////////
// Print UART0/UART1 baud rate divisor register /////////////////////////////////
/////////////////////////////////////////////////////////////////////////////////

static int PrintUARTBRD(int num)
{
 U32 uartbrd;				// UARTBRD0/UARTBRD1 content
 U32 CNT0;				// counter 0
 U32 CNT1;				// counter 1
 U32 brd;				// result baudrate 

 if(num == 0)
  {					// read UARTBRD0 register 
   uartbrd = UARTBRD0;
  } 
 else
 if(num == 1)
  {
   uartbrd = UARTBRD1;			// read UARTBRD1 register 
  }
 else
  {
   return(0);
  } 

 Print("\nUARTBRD%d UART%d Baud Rate Divisor Register = 0x%08x",
   num		,
   num 		,
   uartbrd
  );

//  
 Print("\n3:0  Baud rate divisor value CNT1......%d%", (uartbrd & 0xF));
 if((uartbrd & 0x01) != 0)
  {				
   CNT1 = 16;
   Print("(divide by 1)");
  }
 else
  {
   CNT1  = 1;
   Print("(divide by 16)");
  }
 
 CNT0 = ((uartbrd >> 4) & 0xFFF);
 Print("\n15:4 Time constant value for CNT0......%d", CNT0);

// 
 brd = MCLK2 / (CNT0 + 1) / CNT1 / 16;
 Print("\nBRGOUT = MCLK2/(CNT0+1)/(16^CNT1)/16...%d", brd);
 
//
 return(1);
} 

/////////////////////////////////////////////////////////////////////////////////
// set UARTBRD0/UARTBRD1 register ///////////////////////////////////////////////
/////////////////////////////////////////////////////////////////////////////////

void SetUARTBRD(int num)
{
 U32 uartbrd;				// UARTBRD0/UARTBRD1 content
 U32 CNT0;				// counter 0
 U32 CNT1;				// counter 1

// 
 U32 brdmin;
 U32 brddif;
 U32 brdout;				
 U32 c2;
 U32 c1;
 U32 c0;
 U8 ch;

 if(num == 0)
  {
   uartbrd = UARTBRD0;			// 
  }
 else
 if(num == 1)
  {
   uartbrd = UARTBRD1;			// 
  }
 else 
  {
   return;
  }

// 
 CNT1 = uartbrd & 0x1;			// counter 1
 if(CNT1 == 0) { CNT1 = 1; }		// 0 => divide 1
 else          { CNT1 = 16;}  		// 1 => divide 16
 CNT0 = (uartbrd >> 4) & 0x3FF;		// counter 0
 uartbrd = MCLK2 / (CNT0+1) / CNT1 / 16;// 

// 
 Print("\nSet UART%d baudrate", num); 
 Print("\nCurrent UART%d baud rate....%d", uartbrd);
 Print("\nInput new baud rate (38400).");
 uartbrd = get_number(0, 16);
 if(uartbrd == 0)
  {
   uartbrd = 38400;
  } 

// 
 brdmin = 0xFFFFFFFF; 
 for(c0 = 0; c0 < 4096; ++c0)		// c0 loop
  {
   for(c2 = 0; c2 <= 1; ++c2)		// c1 loop
    {
     switch(c2)
      {
       case  0 : c1 =  1; break;	// divide by  1
       case  1 : c1 = 16; break; 	// divide by 16
       default : break;
      }


// calculate baud rate /////////////////////////////////////////////////////////

     brdout = MCLK2 / (c0 + 1) / c1 / 16;

//
     if(brdout >= uartbrd)
      {					
       brddif = brdout - uartbrd;	// 
      }
     else
      {
       brddif = uartbrd - brdout;	// 	
      }     

     if(brddif < brdmin)
      {
// 
       brdmin = brddif;			// set min difference 
       CNT0 = c0;			// set min cnt0 counter  
       CNT1 = c2;			// set min cnt1 counter 
      }
    }
  }

 while(1)
  {
// 
   Print("\nCNT0......%d", CNT0); 
   Print("\nCNT1......%d", CNT1); 
   uartbrd = ((CNT0 & 0x3FF) << 4) | CNT1;
   Print("\nUARTBRD = 0x%08x", uartbrd);
   if(CNT1 == 0) { c1 =  1; }
   else    	 { c1 = 16; }  

// 
   brdout = MCLK2 / (CNT0 + 1) / c1 / 16;
   Print("\n\nBRGOUT = MCLK2 / (CNT0 + 1) / 16 ** CNT1 / 16");
   Print("\nBRGOUT....%d", brdout);

//    
   Print("\n[0] Change CNT0."); 
   Print("\n[1] Change CNT1."); 
   Print("\n[S] Set UARTBRD%d.", num); 
   Print("\n[Q] Quit.");
   Print("\n    Select Item");
   ch = get_upper(); 
   if(ch == 'Q')
    {					// exit from set brd menu
     return;				// 
    } 
   else
   if(ch == 'S')
    {
     if(num == 0)
      {					// set UARTBRD0 
       UARTBRD0 = uartbrd;		// register 
      } 
     else
      {					// set UARTBRD1 
       UARTBRD1 = uartbrd;		// register 
      }
//
     return;
    }
   else   
   if(ch == '0')
    {					
     Print("\nInput CNT0 ");		// change CNT0 
     CNT0 = get_number(0, 16);
     if(CNT0 > 4095)
      {					// set CNT0 to
       CNT0 = 4095;			// upper limit 
      } 
    } 
   else
   if(ch == '1')
    {
     Print("\nInput CNT1 ");		// change CNT1 
     CNT1 = get_number(0, 16);
     if(CNT1 != 0 && CNT1 != 1)
      {					// 
       CNT1 = 0;			// 
      }
    }
  }
}


/////////////////////////////////////////////////////////////////////////////////
// View UART0/UART1 Configuration ///////////////////////////////////////////////
/////////////////////////////////////////////////////////////////////////////////

// print UARTLCON register (KS32C50100 only) //////////////////////////////////// 
static void UARTLCONView(void)
{
#ifdef _S3C4530_				// with s3c45 support
 if(is_s3c4530(syscfg_pd_id)) {
  return;
 }
#endif								/* _S3C4530_ */

// UART0/UART1 Line Control registers //////////////////////////////////////////
 PrintUARTLCON(0); 			// UARTLCON0 - UART0 Line Control Register
 PrintUARTLCON(1); 			// UARTLCON1 - UART1 Line Control Register
}


#ifdef _KS32C50100_
/////////////////////////////////////////////////////////////////////////////////
// view & edit S3C4510 UART 0 & 1 control registers /////////////////////////////
/////////////////////////////////////////////////////////////////////////////////
static void S3C4510UCONView(void)
{
 while(1)
  {
   U8 it;
   PrintUARTCONT(0);  	 		// UARTCONT0 - UART0 Control Register
   PrintUARTCONT(1); 			// UARTCONT1 - UART1 Control Register
// 
   Print("\n[1] Toggle UART0 DSR.");
   Print("\n[2] Toggle UART1 DSR.");
   Print("\n[3] UART0 Send Break.");
   Print("\n[4] UART1 Send Break.");
   Print("\n    Quit");
   Print("\nSelect Item :");
   it = get_upper();   
   if(it == 'Q')
    {
     break;
    } 
 
   switch(it)
    {
     case '1' : UARTCONT0 ^= UARTCON_DSR;        break; // Toggle UART0 DSR
     case '2' : UARTCONT1 ^= UARTCON_DSR;        break; // Toggle UART1 DSR
     case '3' : UARTCONT0 ^= UARTCON_SEND_BREAK; break; // UART0 Send Break
     case '4' : UARTCONT1 ^= UARTCON_SEND_BREAK; break; // UART0 Send Break.
     default  :
      break;
    }
  }
}
#endif

#ifdef _S3C4530_
// view & edit S3C4510 UART 0 & 1 control registers /////////////////////////////
static void S3C4530UCONView(void)
{
 PrintUCONT(-2);
}
#endif								/* _S3C4530_ */


void UARTConfigView(void)
{
 while(1)
  {
   U8 it; 
   Print("\n\nUART0 & UART1 Configuration ");

#ifdef _S3C4530_
   if(is_s3c4530(syscfg_pd_id)) {
// s3C4530 specific registers //////////////////////////////////////////////////
	Print("S3C4530");
   	Print("\n[C] UCONT0    - UART0 Control Register."); 
   	Print("\n    UCONT1    - UART1 Control Register."); 
   } else {
	Print("KS32C50100");
   	Print("\n[L] UARTLCON0 - UART0 Line Control Register.");
   	Print("\n    UARTLCON1 - UART1 Line Control Register.");
   	Print("\n[C] UARTCONT0 - UART0 Control Register."); 
   	Print("\n    UARTCONT1 - UART1 Control Register)."); 
   }	
#else
   Print("KS32C50100");
   Print("\n[L] UARTLCON0 - UART0 Line Control Register.");
   Print("\n    UARTLCON1 - UART1 Line Control Register.");
   Print("\n[C] UARTCONT0 - UART0 Control Register."); 
   Print("\n    UARTCONT1 - UART1 Control Register."); 
#endif								/* _S3C4530_ */

   Print("\n[B] UARTBRD0  - UART0 Baud Rate Divisor Register.");
   Print("\n    UARTBRD1  - UART0 Baud Rate Divisor Register.");
#ifdef _S3C4530_
   if(is_s3c4530(syscfg_pd_id)) {
   Print("\n[S] USTAT0    - UART0 Status Register.");
   Print("\n    USTAT1    - UART1 Status Register.");
   Print("\n[I] UINTEN0   - UART0 Int Enable Register.");
   Print("\n    UINTEN1   - UART1 Int Enable Register.");
   } else {
   Print("\n[S] UARTSTAT0 - UART0 Status Register.");
   Print("\n    UARTSTAT1 - UART1 Status Register.");
   }
#else 
   Print("\n[S] UARTSTAT0 - UART0 Status Register.");
   Print("\n    UARTSTAT1 - UART1 Status Register.");
#endif								/* _S3C4530_ */
   Print("\n[Q] Quit");
   Print("\nSelect Item :");

   it = get_upper();
   switch(it) 
    {
     case 'L' : 
       UARTLCONView(); 
       break;

     case 'C' : 
////////////////////////////////////////////////////////////////////////////////
// UART0/UART1 Control registers /////////////////////////////////////////////// 
////////////////////////////////////////////////////////////////////////////////
#ifdef _S3C4530_ 
	if(is_s3c4530(syscfg_pd_id)) {
	  S3C4530UCONView();		
	} else {
	  S3C4510UCONView();		
	}
#else
	  S3C4510UCONView();		
#endif								/* _S3C4530_ */
      break; 

     case 'B' : 
////////////////////////////////////////////////////////////////////////////////
// UART0/UART1 Baud Rate Divisor Register ////////////////////////////////////// 
////////////////////////////////////////////////////////////////////////////////
      while(1)
       {
        PrintUARTBRD(0); 	// UARTBRD0  - UART0 Baud Rate Divisor Register
        PrintUARTBRD(1);  	// UARTBRD1  - UART0 Baud Rate Divisor Register
        Print("\n[0] Set UART0 baudrate.");
        Print("\n[1] Set UART1 baudrate.");
        Print("\n    Quit");
        Print("\nSelect Item :");
        it = get_upper();   
        if(it == 'Q')
         {
          break;
         } 
 
        switch(it)
         {
          case '0' : SetUARTBRD(0); break; // Set UARTBRD0 
          case '1' : SetUARTBRD(1); break; // Set UARTBRD1 
          default  :
           break;
         }
       }
      break; 
 
     case 'S' : 
// UART0/UART1 Status register ///////////////////////////////////////////////// 
#ifdef _S3C4530_
      if(is_s3c4530(syscfg_pd_id)) {
        PrintUSTAT(-2);			// print USTAT0 & USTAT1 
      } else {
        PrintUARTSTAT(0); 		// UARTSTAT0 - UART0 Status Register
        PrintUARTSTAT(1); 		// UARTSTAT1 - UART1 Status Register
      }	
#else       	
      PrintUARTSTAT(0); 		// UARTSTAT0 - UART0 Status Register
      PrintUARTSTAT(1); 		// UARTSTAT1 - UART1 Status Register
#endif 								/* _S3C4530_ */
      break; 

#ifdef _S3C4530_
// print S3C4530 interrupt enable register /////////////////////////////////////
     case 'I' : 
	if(is_s3c4530(syscfg_pd_id)) {
     	  PrintUINTEN(-2);		// UART 0 int enable register 
	}
	break;
#endif 								// _S3C4530_
	
     case 'Q' : 
      return;
    }

   Print("\nPress any key to continue.\n");
   it = get_upper();
   if(it == 'Q')
    {
     return;
    } 
  }  
}


////////////////////////////////////////////////////////////////////////////////
// UART TEST TOP MODULE ////////////////////////////////////////////////////////
////////////////////////////////////////////////////////////////////////////////

void UartTest(void)
{
// 
 UART_Initialize();			// reinit UART
// 
 while(1) 
  {
   U8 it;
   Print("\n\nUART 0&1 TEST") ;
   Print("\n[C] UART0 & UART1 configurations view & change.");
   Print("\n[0] URRT0 Internal Loopback test(POLL).");
   Print("\n[1] URRT1 Internal Loopback test(POLL).");
   Print("\n[2] URRT0 Internal Loopback test(INTERRUPT).");
   Print("\n[3] URRT1 Internal Loopback test(INTERRUPT).");
   Print("\n[4] URRT0 & UART1 Internal Loopback test(INTERRUPT).");
   Print("\n[5] UART0 String echo (INTERRUPT).");
   Print("\n[6] UART1 String echo (INTERRUPT).");
   Print("\n[7] UART0 External Loopback (INTERRUPT).");
   Print("\n[8] UART1 External Loopback (INTERRUPT).");
   Print("\n[Q] Quit - Return to main menu.");
   Print("\nSelect Item :");

// 
   it = get_upper();
   switch(it) 
    {
     case 'C' : UARTConfigView();  break;  // Print UART0/UART1 Configuration
     case '0' : UARTAutoLoopBack(0); break;// UART0 LoopBack test
     case '1' : UARTAutoLoopBack(1); break;// UART1 LoopBack test
     case '2' : UARTLoopBackInt(0); break; // UART0 Int LoopBack test
     case '3' : UARTLoopBackInt(1); break; // UART1 Int LoopBack test
     case '4' : UARTIntLoopBack(); break;  // UART1 Int LoopBack test
     case '5' : StringEcho(0); break; 	   // UART0 String Echo
     case '6' : StringEcho(1); break; 	   // UART1 String Echo
     case '7' : UARTExtLoopBackInt(0);break;// UART0 External Loopback test	
     case '8' : UARTExtLoopBackInt(1);break;// UART1 External Loopback test	
     case 'Q' : return;
     default  : 
      break;
    }
//    
   Print("\nPress any key to continue.\n");
   it = get_byte();
   if(it == 'Q')
    {
     return;
    } 
  } 
}
